@@ -785,7 +785,7 @@ XLALSimIMREOBNRv2Generator(
    LIGOTimeGPS             epoch = LIGOTIMEGPSZERO;
 
    /* Variables for the integrator */
-   ark4GSLIntegrator       *integrator = NULL;
+   LALAdaptiveRungeKutta4Integrator       *integrator = NULL;
    REAL8Array              *dynamics   = NULL;
    REAL8Array              *dynamicsHi = NULL;
    INT4                    retLen;
