@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013,2014 Michael Puerrer, Alejandro Bohe
+ *  Copyright (C) 2013,2014,2015 Michael Puerrer, Alejandro Bohe
  *  Reuses code found in:
  *    - LALSimIMRPhenomC
  *    - LALSimInspiralSpinTaylorF2
@@ -31,6 +31,9 @@
 
 #include <stdlib.h>
 #include <math.h>
+#include <gsl/gsl_errno.h>
+#include <gsl/gsl_spline.h>
+
 #include <lal/Date.h>
 #include <lal/FrequencySeries.h>
 #include <lal/LALAtomicDatatypes.h>
@@ -40,9 +43,16 @@
 #include <lal/Units.h>
 #include <lal/XLALError.h>
 #include <lal/SphericalHarmonics.h>
+#include <lal/Sequence.h>
+
 #include "LALSimIMR.h"
 #include "LALSimIMRPhenomC_internals.c" /* This is ugly, but allows us to reuse internal PhenomC functions without making those functions XLAL */
 
+#ifndef _OPENMP
+#define omp ignore
+#endif
+
+
 /**************************** PhenomP internal function prototypes *****************************/
 
 /* PhenomC parameters for modified ringdown: Uses final spin formula of Barausse & Rezzolla, Astrophys.J.Lett.704:L40-L44, 2009 */
@@ -78,8 +88,28 @@ typedef struct tagSpinWeightedSphericalHarmonic_l2 {
   COMPLEX16 Y2m2, Y2m1, Y20, Y21, Y22;
 } SpinWeightedSphericalHarmonic_l2;
 
+/* Internal core function to calculate PhenomP polarizations for a sequence of frequences. */
+static int PhenomPCore(
+  COMPLEX16FrequencySeries **hptilde,   /**< Output: Frequency-domain waveform h+ */
+  COMPLEX16FrequencySeries **hctilde,   /**< Output: Frequency-domain waveform hx */
+  const REAL8 chi_eff,                  /**< Effective aligned spin */
+  const REAL8 chip,                     /**< Effective spin in the orbital plane */
+  const REAL8 eta,                      /**< Symmetric mass-ratio */
+  const REAL8 thetaJ,                   /**< Angle between J0 and line of sight (z-direction) */
+  const REAL8 Mtot_SI,                  /**< Total mass of binary (kg) */
+  const REAL8 distance,                 /**< Distance of source (m) */
+  const REAL8 alpha0,                   /**< Initial value of alpha angle (azimuthal precession angle) */
+  const REAL8 phic,                     /**< Orbital phase at the peak of the underlying non precessing model (rad) */
+  const REAL8 f_ref,                    /**< Reference frequency */
+  const REAL8Sequence *freqs,           /**< Frequency points at which to evaluate the waveform (Hz) */
+  double deltaF
+  /**< If deltaF > 0, the frequency points given in freqs are uniformly spaced with
+   * spacing deltaF. Otherwise, the frequency points are spaced non-uniformly.
+   * Then we will use deltaF = 0 to create the frequency series we return. */
+);
+
 /* Internal core function to calculate PhenomP polarizations for a single frequency. */
-int PhenomPCore(
+static int PhenomPCoreOneFrequency(
   const REAL8 fHz,                        /**< Frequency (Hz) */
   const REAL8 eta,                        /**< Symmetric mass ratio */
   const REAL8 chi_eff,                    /**< Dimensionless effective total aligned spin */
@@ -92,8 +122,9 @@ int PhenomPCore(
   SpinWeightedSphericalHarmonic_l2 *Y2m,  /**< Struct of l=2 spherical harmonics of spin weight -2 */
   const REAL8 alphaoffset,                /**< f_ref dependent offset for alpha angle */
   const REAL8 epsilonoffset,              /**< f_ref dependent offset for epsilon angle */
-  COMPLEX16 *hp,                          /**< Output: \tilde h_+ */
-  COMPLEX16 *hc                           /**< Output: \tilde h_+ */
+  COMPLEX16 *hp,                          /**< Output: \f$\tilde h_+\f$ */
+  COMPLEX16 *hc,                          /**< Output: \f$\tilde h_+\f$ */
+  REAL8 *phasing                          /**< Output: overall phasing */
 );
 
 /* Simple 2PN version of L, without any spin terms expressed as a function of v */
@@ -127,14 +158,13 @@ REAL8 FinalSpinBarausse2009(  /* Barausse & Rezzolla, Astrophys.J.Lett.704:L40-L
   const REAL8 nu,               /**< Symmetric mass-ratio */
   const REAL8 a1,               /**< |a_1| norm of dimensionless spin vector for BH 1 */
   const REAL8 a2,               /**< |a_2| norm of dimensionless spin vector for BH 2 */
-  const REAL8 cos_alpha,        /**< cos(alpha) = \hat a_1 . \hat a_2 (Eq. 7) */
-  const REAL8 cos_beta_tilde,   /**< cos(\tilde beta)  = \hat a_1 . \hat L (Eq. 9) */
-  const REAL8 cos_gamma_tilde   /**< cos(\tilde gamma) = \hat a_2 . \hat L (Eq. 9)*/
+  const REAL8 cos_alpha,        /**< \f$cos(alpha) = \hat a_1 . \hat a_2\f$ (Eq. 7) */
+  const REAL8 cos_beta_tilde,   /**< \f$cos(\tilde beta)  = \hat a_1 . \hat L\f$ (Eq. 9) */
+  const REAL8 cos_gamma_tilde   /**< \f$cos(\tilde gamma) = \hat a_2 . \hat L\f$ (Eq. 9)*/
 );
 
 static size_t NextPow2(const size_t n); /* Return the closest higher power of 2. */
 
-
 /*************************************** Implementation ****************************************/
 
 int XLALSimIMRPhenomPCalculateModelParameters(
@@ -142,8 +172,7 @@ int XLALSimIMRPhenomPCalculateModelParameters(
     REAL8 *chip,                    /**< Output: Effective spin in the orbital plane */
     REAL8 *eta,                     /**< Output: Symmetric mass-ratio */
     REAL8 *thetaJ,                  /**< Output: Angle between J0 and line of sight (z-direction) */
-    REAL8 *phiJ,                    /**< Output: Angle of J0 in the plane of the sky */
-    REAL8 *alpha0,                  /**< Output: Initial value of alpha angle */
+    REAL8 *alpha0,                  /**< Output: Initial value of alpha angle (azimuthal precession angle) */
     const REAL8 m1_SI,              /**< Mass of companion 1 (kg) */
     const REAL8 m2_SI,              /**< Mass of companion 2 (kg) */
     const REAL8 f_ref,              /**< Reference GW frequency (Hz) */
@@ -157,18 +186,18 @@ int XLALSimIMRPhenomPCalculateModelParameters(
     const REAL8 s2y,                /**< Initial value of s2y: dimensionless spin of BH 2 */
     const REAL8 s2z)                /**< Initial value of s2z: dimensionless spin of BH 2 */
 {
-  /* Check inputs for sanity */
+  // See Fig. 1. in arxiv:1408.1810 for diagram of the angles.
+  // Note that the angle phiJ defined below and alpha0 are degenerate. Therefore we do not output phiJ.
+
+  /* Check arguments for sanity */
   if (!chi_eff)  XLAL_ERROR(XLAL_EFAULT);
   if (!chip)     XLAL_ERROR(XLAL_EFAULT);
   if (!eta)      XLAL_ERROR(XLAL_EFAULT);
   if (!thetaJ)   XLAL_ERROR(XLAL_EFAULT);
-  if (!phiJ)     XLAL_ERROR(XLAL_EFAULT);
   if (!alpha0)   XLAL_ERROR(XLAL_EFAULT);
 
-  if (f_ref <= 0) {
-    XLALPrintError("Reference frequency must be positive.\n");
-    XLAL_ERROR(XLAL_EDOM);
-  }
+  if (f_ref <= 0)
+    XLAL_ERROR(XLAL_EDOM, "Reference frequency must be positive.\n");
 
   const REAL8 m1 = m1_SI / LAL_MSUN_SI;   /* Masses in solar masses */
   const REAL8 m2 = m2_SI / LAL_MSUN_SI;
@@ -213,10 +242,11 @@ int XLALSimIMRPhenomPCalculateModelParameters(
   const REAL8 J0 = sqrt(Jx0*Jx0 + Jy0*Jy0 + Jz0*Jz0);
 
   *thetaJ = acos(Jz0 / J0); /* Angle between J0 and line of sight (z-direction) */
+  REAL8 phiJ; // We only use this angle internally since it is degenerate with alpha0.
   if (Jx0 == 0.0 && Jy0 == 0.0)
-    *phiJ = 0;
+    phiJ = 0;
   else
-    *phiJ = atan2(Jy0, Jx0); /* Angle of J0 in the plane of the sky */
+    phiJ = atan2(Jy0, Jx0); /* Angle of J0 in the plane of the sky */
     /* Note: Compared to the similar code in SpinTaylorF2 we have defined phiJ as the angle between the positive
     (rather than the negative) x-axis and the projection of J0, since this is a more natural definition of the angle.
     We have also renamed the angle from psiJ to phiJ. */
@@ -229,8 +259,8 @@ int XLALSimIMRPhenomPCalculateModelParameters(
       {cos(phiJ)*sin(thetaJ), sin(thetaJ)*sin(phiJ),cos(thetaJ)}
     }
   */
-  const REAL8 rotLx = -lnhatx*cos(*thetaJ)*cos(*phiJ) - lnhaty*cos(*thetaJ)*sin(*phiJ) + lnhatz*sin(*thetaJ);
-  const REAL8 rotLy = lnhatx*sin(*phiJ) - lnhaty*cos(*phiJ);
+  const REAL8 rotLx = -lnhatx*cos(*thetaJ)*cos(phiJ) - lnhaty*cos(*thetaJ)*sin(phiJ) + lnhatz*sin(*thetaJ);
+  const REAL8 rotLy = lnhatx*sin(phiJ) - lnhaty*cos(phiJ);
   if (rotLx == 0.0 && rotLy == 0.0)
     *alpha0 = 0.0;
   else
@@ -246,16 +276,89 @@ int XLALSimIMRPhenomP(
   const REAL8 chip,                     /**< Effective spin in the orbital plane */
   const REAL8 eta,                      /**< Symmetric mass-ratio */
   const REAL8 thetaJ,                   /**< Angle between J0 and line of sight (z-direction) */
-  const REAL8 phiJ,                     /**< Angle of J0 in the plane of the sky */
   const REAL8 Mtot_SI,                  /**< Total mass of binary (kg) */
   const REAL8 distance,                 /**< Distance of source (m) */
-  const REAL8 alpha0,                   /**< Initial value of alpha angle */
+  const REAL8 alpha0,                   /**< Initial value of alpha angle (azimuthal precession angle) */
   const REAL8 phic,                     /**< Orbital phase at the peak of the underlying non precessing model (rad) */
   const REAL8 deltaF,                   /**< Sampling frequency (Hz) */
   const REAL8 f_min,                    /**< Starting GW frequency (Hz) */
   const REAL8 f_max,                    /**< End frequency; 0 defaults to ringdown cutoff freq */
   const REAL8 f_ref)                    /**< Reference frequency */
 {
+  // See Fig. 1. in arxiv:1408.1810 for diagram of the angles.
+  // Note that the angles phiJ which is calculated internally in XLALSimIMRPhenomPCalculateModelParameters
+  // and alpha0 are degenerate. Therefore phiJ is not passed to this function.
+
+  // Use fLow, fHigh, deltaF to compute freqs sequence
+  // Instead of building a full sequency we only transfer the boundaries and let
+  // the internal core function do the rest (and properly take care of corner cases).
+  REAL8Sequence *freqs = XLALCreateREAL8Sequence(2);
+  freqs->data[0] = f_min;
+  freqs->data[1] = f_max;
+
+  int retcode = PhenomPCore(hptilde, hctilde,
+      chi_eff, chip, eta, thetaJ, Mtot_SI, distance, alpha0, phic, f_ref, freqs, deltaF);
+
+  return (retcode);
+}
+
+int XLALSimIMRPhenomPFrequencySequence(
+  COMPLEX16FrequencySeries **hptilde,   /**< Output: Frequency-domain waveform h+ */
+  COMPLEX16FrequencySeries **hctilde,   /**< Output: Frequency-domain waveform hx */
+  const REAL8Sequence *freqs,           /**< Frequency points at which to evaluate the waveform (Hz) */
+  const REAL8 chi_eff,                  /**< Effective aligned spin */
+  const REAL8 chip,                     /**< Effective spin in the orbital plane */
+  const REAL8 eta,                      /**< Symmetric mass-ratio */
+  const REAL8 thetaJ,                   /**< Angle between J0 and line of sight (z-direction) */
+  const REAL8 Mtot_SI,                  /**< Total mass of binary (kg) */
+  const REAL8 distance,                 /**< Distance of source (m) */
+  const REAL8 alpha0,                   /**< Initial value of alpha angle (azimuthal precession angle) */
+  const REAL8 phic,                     /**< Orbital phase at the peak of the underlying non precessing model (rad) */
+  const REAL8 f_ref)                    /**< Reference frequency */
+{
+  // See Fig. 1. in arxiv:1408.1810 for diagram of the angles.
+  // Note that the angles phiJ which is calculated internally in XLALSimIMRPhenomPCalculateModelParameters
+  // and alpha0 are degenerate. Therefore phiJ is not passed to this function.
+
+  // Call the internal core function with deltaF = 0 to indicate that freqs is non-uniformly
+  // spaced and we want the strain only at these frequencies
+  int retcode = PhenomPCore(hptilde, hctilde,
+      chi_eff, chip, eta, thetaJ, Mtot_SI, distance, alpha0, phic, f_ref, freqs, 0);
+
+  return(retcode);
+}
+
+/* Internal core function to calculate PhenomP polarizations for a sequence of frequences. */
+static int PhenomPCore(
+  COMPLEX16FrequencySeries **hptilde,   /**< Output: Frequency-domain waveform h+ */
+  COMPLEX16FrequencySeries **hctilde,   /**< Output: Frequency-domain waveform hx */
+  const REAL8 chi_eff,                  /**< Effective aligned spin */
+  const REAL8 chip,                     /**< Effective spin in the orbital plane */
+  const REAL8 eta,                      /**< Symmetric mass-ratio */
+  const REAL8 thetaJ,                   /**< Angle between J0 and line of sight (z-direction) */
+  const REAL8 Mtot_SI,                  /**< Total mass of binary (kg) */
+  const REAL8 distance,                 /**< Distance of source (m) */
+  const REAL8 alpha0,                   /**< Initial value of alpha angle (azimuthal precession angle) */
+  const REAL8 phic,                     /**< Orbital phase at the peak of the underlying non precessing model (rad) */
+  const REAL8 f_ref,                    /**< Reference frequency */
+  const REAL8Sequence *freqs_in,        /**< Frequency points at which to evaluate the waveform (Hz) */
+  double deltaF
+  /* If deltaF > 0, the frequency points given in freqs are uniformly spaced with
+   * spacing deltaF. Otherwise, the frequency points are spaced non-uniformly.
+   * Then we will use deltaF = 0 to create the frequency series we return. */
+  )
+{
+  // See Fig. 1. in arxiv:1408.1810 for diagram of the angles.
+  // Note that the angles phiJ which is calculated internally in XLALSimIMRPhenomPCalculateModelParameters
+  // and alpha0 are degenerate. Therefore phiJ is not passed to this function.
+
+  XLAL_PRINT_INFO("*** PhenomPCore() ***");
+  
+  /* Find frequency bounds */
+  if (!freqs_in) XLAL_ERROR(XLAL_EFAULT);
+  double f_min = freqs_in->data[0];
+  double f_max = freqs_in->data[freqs_in->length - 1];
+
   const REAL8 M = Mtot_SI / LAL_MSUN_SI;  /* External units: SI; internal units: solar masses */
   const REAL8 m_sec = M * LAL_MTSUN_SI;   /* Total mass in seconds */
   const REAL8 q = (1.0 + sqrt(1.0 - 4.0*eta) - 2.0*eta)/(2.0*eta); /* Mass-ratio */
@@ -264,39 +367,32 @@ int XLALSimIMRPhenomP(
   const REAL8 piM = LAL_PI * m_sec;
   const REAL8 v0 = cbrt(piM * f_ref);
 
-  static LIGOTimeGPS ligotimegps_zero = {0, 0};
+  LIGOTimeGPS ligotimegps_zero = LIGOTIMEGPSZERO; // = {0, 0}
 
   /* Check inputs for sanity */
   if (*hptilde)       XLAL_ERROR(XLAL_EFAULT);
   if (*hctilde)       XLAL_ERROR(XLAL_EFAULT);
-  if (deltaF <= 0)    XLAL_ERROR(XLAL_EDOM);
+  if (deltaF < 0)    XLAL_ERROR(XLAL_EDOM);
   if (M <= 0)         XLAL_ERROR(XLAL_EDOM);
   if (f_min <= 0)     XLAL_ERROR(XLAL_EDOM);
   if (f_max < 0)      XLAL_ERROR(XLAL_EDOM);
   if (distance <= 0)  XLAL_ERROR(XLAL_EDOM);
 
-  if (f_ref <= 0) {
-      XLALPrintError("Reference frequency must be positive.\n");
-      XLAL_ERROR(XLAL_EDOM);
-  }
+  if (f_ref <= 0)
+      XLAL_ERROR(XLAL_EDOM, "Reference frequency must be positive.\n");
 
-  if (eta < 0.0453515){ /* q = 20 */
-      XLALPrintError("Mass ratio is way outside the calibration range. m1/m2 should be <= 20.\n");
-      XLAL_ERROR(XLAL_EDOM);
-  }
-  else if (eta < 0.16) { /* q = 4 */
-      XLALPrintWarning("Warning: The model is only calibrated for m1/m2 <= 4.\n");
-  }
-
-  if (fabs(chi_eff) > 1) XLAL_ERROR(XLAL_EDOM);
-  if (fabs(chip) > 1) XLAL_ERROR(XLAL_EDOM);
+  if (eta < 0.0453515) /* q = 20 */
+      XLAL_ERROR(XLAL_EDOM, "Mass ratio is way outside the calibration range. m1/m2 should be <= 20.\n");
+  else if (eta < 0.16)  /* q = 4 */
+      XLAL_PRINT_WARNING("Warning: The model is only calibrated for m1/m2 <= 4.\n");
 
   /* If spins are above 0.9 or below -0.9, throw an error. */
   /* The rationale behind this is given at this page: https://www.lsc-group.phys.uwm.edu/ligovirgo/cbcnote/WaveformsReview/IMRPhenomCdevel-SanityCheck01 */
-  if (chi_eff > 0.9 || chi_eff < -0.9){
-      XLALPrintError("Spins outside the range [-0.9,0.9] are not supported\n");
-      XLAL_ERROR(XLAL_EDOM);
-  }
+  if (chi_eff > 0.9 || chi_eff < -0.9)
+      XLAL_ERROR(XLAL_EDOM, "Effective spin chi_eff = %g outside the range [-0.9,0.9] is not supported!\n", chi_eff);
+
+  if (fabs(chip) > 1)
+    XLAL_ERROR(XLAL_EDOM, "In-plane spin chip =%g is super extremal!\n", chip);
 
   const REAL8 chil = (1.0+q)/q * chi_eff; /* dimensionless aligned spin of the largest BH */
   NNLOanglecoeffs angcoeffs;
@@ -319,7 +415,7 @@ int XLALSimIMRPhenomP(
                                 + angcoeffs.epsiloncoeff4*logomega_ref
                                 + angcoeffs.epsiloncoeff5*omega_ref_cbrt);
 
-  /* Compute Ylm's only once and pass them to PhenomPCore() below. */
+  /* Compute Ylm's only once and pass them to PhenomPCoreOneFrequency() below. */
   SpinWeightedSphericalHarmonic_l2 Y2m;
   const REAL8 ytheta  = thetaJ;
   const REAL8 yphi    = 0;
@@ -335,27 +431,67 @@ int XLALSimIMRPhenomP(
   //PCparams = ComputeIMRPhenomCParams(m1, m2, chi_eff); /* original PhenomC */
   PCparams = ComputeIMRPhenomCParamsRDmod(m1, m2, chi_eff, chip); /* PhenomC with ringdown using Barausse 2009 formula for final spin */
   if (!PCparams) XLAL_ERROR(XLAL_EFUNC);
-  if (PCparams->fCut <= f_min) {
-      XLALPrintError("(fCut = 0.15M) <= f_min\n");
-      XLAL_ERROR(XLAL_EDOM);
-  }
+  if (PCparams->fCut <= f_min)
+      XLAL_ERROR(XLAL_EDOM, "(fCut = 0.15M) <= f_min\n");
 
   /* Default f_max to params->fCut */
   REAL8 f_max_prime = f_max ? f_max : PCparams->fCut;
   f_max_prime = (f_max_prime > PCparams->fCut) ? PCparams->fCut : f_max_prime;
-  if (f_max_prime <= f_min) {
-      XLALPrintError("f_max <= f_min\n");
-      XLAL_ERROR(XLAL_EDOM);
-  }
-  XLALPrintInfo("f_max_prime = %g\tfcut = %g\tv = %g\n", f_max_prime, PCparams->fCut, cbrt(piM * f_max_prime));
+  if (f_max_prime <= f_min)
+      XLAL_ERROR(XLAL_EDOM, "f_max <= f_min\n");
+  XLAL_PRINT_INFO("f_max_prime = %g\tfcut = %g\tv = %g\n", f_max_prime, PCparams->fCut, cbrt(piM * f_max_prime));
 
 
   /* Allocate hp, hc */
-  XLALPrintInfo("f_max / deltaF = %g\n", f_max_prime / deltaF);
-  size_t n = NextPow2(f_max_prime / deltaF) + 1; /* Note: Should explain why the length is one plus a power of 2. */
-  XLALPrintInfo("n = %d\n", (int)(n));
-  *hptilde = XLALCreateCOMPLEX16FrequencySeries("hptilde: FD waveform", &ligotimegps_zero, 0.0, deltaF, &lalStrainUnit, n);
-  *hctilde = XLALCreateCOMPLEX16FrequencySeries("hctilde: FD waveform", &ligotimegps_zero, 0.0, deltaF, &lalStrainUnit, n);
+  XLAL_PRINT_INFO("f_max / deltaF = %g\n", f_max_prime / deltaF);
+
+  UINT4 L_fCut = 0; // number of frequency points before we hit fCut
+  size_t n = 0;
+  UINT4 offset = 0; // Index shift between freqs and the frequency series
+  REAL8Sequence *freqs = NULL;
+  if (deltaF > 0)  { // freqs contains uniform frequency grid with spacing deltaF; we start at frequency 0
+    /* Set up output array with size closest power of 2 */
+    n = NextPow2(f_max_prime / deltaF) + 1;
+    if (f_max_prime < f_max)                       /* Resize waveform if user wants f_max larger than cutoff frequency */
+      n = NextPow2(f_max / deltaF) + 1;
+
+    /* coalesce at t=0 */
+    XLALGPSAdd(&ligotimegps_zero, -1. / deltaF); // shift by overall length in time
+
+    *hptilde = XLALCreateCOMPLEX16FrequencySeries("hptilde: FD waveform", &ligotimegps_zero, 0.0, deltaF, &lalStrainUnit, n);
+    *hctilde = XLALCreateCOMPLEX16FrequencySeries("hctilde: FD waveform", &ligotimegps_zero, 0.0, deltaF, &lalStrainUnit, n);
+
+    // Recreate freqs using only the lower and upper bounds
+    size_t i_min = (size_t) (f_min / deltaF);
+    size_t i_max = (size_t) (f_max_prime / deltaF);
+    freqs = XLALCreateREAL8Sequence(i_max - i_min);
+    for (UINT4 i=i_min; i<i_max; i++, L_fCut++)
+      freqs->data[i-i_min] = i*deltaF;
+
+    offset = i_min;
+  } else { // freqs contains frequencies with non-uniform spacing; we start at lowest given frequency
+    n = freqs_in->length;
+
+    *hptilde = XLALCreateCOMPLEX16FrequencySeries("hptilde: FD waveform", &ligotimegps_zero, f_min, 0, &lalStrainUnit, n);
+    *hctilde = XLALCreateCOMPLEX16FrequencySeries("hctilde: FD waveform", &ligotimegps_zero, f_min, 0, &lalStrainUnit, n);
+    offset = 0;
+
+    // Enforce that FS is strictly increasing
+    // (This is needed for phase correction towards the end of this function.)
+    for (UINT4 i=1; i<n; i++)
+      if (!(freqs_in->data[i] > freqs_in->data[i-1]))
+        XLAL_ERROR(XLAL_EDOM, "Frequency sequence must be strictly increasing!\n");
+
+    freqs = XLALCreateREAL8Sequence(n);
+    // Restrict sequence to frequencies <= fCut
+    for (UINT4 i=0; i<n; i++)
+      if (freqs_in->data[i] <= PCparams->fCut) {
+        freqs->data[i] = freqs_in->data[i];
+        L_fCut++;
+      }
+  }
+
+
   memset((*hptilde)->data->data, 0, n * sizeof(COMPLEX16));
   memset((*hctilde)->data->data, 0, n * sizeof(COMPLEX16));
   XLALUnitMultiply(&((*hptilde)->sampleUnits), &((*hptilde)->sampleUnits), &lalSecondUnit);
@@ -364,29 +500,32 @@ int XLALSimIMRPhenomP(
     XLAL_ERROR(XLAL_EFUNC);
 
   /* Test output */
-  XLALPrintInfo("eta: %g\n", eta);
-  XLALPrintInfo("m1: %g\n", m1);
-  XLALPrintInfo("m2: %g\n", m2);
-  XLALPrintInfo("chi: %g\n", chi_eff);
-  XLALPrintInfo("chip: %g\n", chip);
-  XLALPrintInfo("thetaJ: %g\n", thetaJ);
-  XLALPrintInfo("phiJ: %g\n", phiJ);
-  XLALPrintInfo("ind_mix = (int)(f_mix / deltaF) = %d\n", (int)(f_min / deltaF));
-  XLALPrintInfo("ind_max = (int)(f_max / deltaF) = %d\n", (int)(f_max_prime / deltaF));
+  XLAL_PRINT_INFO("eta: %g\n", eta);
+  XLAL_PRINT_INFO("m1: %g\n", m1);
+  XLAL_PRINT_INFO("m2: %g\n", m2);
+  XLAL_PRINT_INFO("chi: %g\n", chi_eff);
+  XLAL_PRINT_INFO("chip: %g\n", chip);
+  XLAL_PRINT_INFO("thetaJ: %g\n", thetaJ);
+  XLAL_PRINT_INFO("ind_mix = (int)(f_mix / deltaF) = %d\n", (int)(f_min / deltaF));
+  XLAL_PRINT_INFO("ind_max = (int)(f_max / deltaF) = %d\n", (int)(f_max_prime / deltaF));
 
-  /* Note: there will usually be zero data at the beginning and end of the frequency series  */
-  size_t i_min = (size_t) (f_min / deltaF);
-  size_t i_max = (size_t) (f_max_prime / deltaF);
   int errcode = XLAL_SUCCESS;
+
+  REAL8 *phis = XLALMalloc(L_fCut*sizeof(REAL8)); // array for waveform phase
+  REAL8 phasing = 0;
+
   /*
     We can't call XLAL_ERROR() directly with OpenMP on.
     Keep track of return codes for each thread and in addition use flush to get out of
     the parallel for loop as soon as possible if something went wrong in any thread.
   */
   #pragma omp parallel for
-  for (size_t i = i_min; i < i_max; i++) {
-    COMPLEX16 hp_val, hc_val;
-    REAL8 f = i * deltaF;
+  for (UINT4 i=0; i<L_fCut; i++) { // loop over frequency points in sequence
+    COMPLEX16 hp_val = 0.0;
+    COMPLEX16 hc_val = 0.0;
+    double f = freqs->data[i];
+    int j = i + offset; // shift index for frequency series if needed
+
     int per_thread_errcode;
 
     #pragma omp flush(errcode)
@@ -394,23 +533,62 @@ int XLALSimIMRPhenomP(
       goto skip;
 
     /* Generate the waveform */
-    per_thread_errcode = PhenomPCore(f, eta, chi_eff, chip, distance, M, phic,
+    per_thread_errcode = PhenomPCoreOneFrequency(f, eta, chi_eff, chip, distance, M, phic,
                               PCparams, &angcoeffs, &Y2m,
                               alphaNNLOoffset - alpha0, epsilonNNLOoffset,
-                              &hp_val, &hc_val);
+                              &hp_val, &hc_val, &phasing);
 
     if (per_thread_errcode != XLAL_SUCCESS) {
       errcode = per_thread_errcode;
       #pragma omp flush(errcode)
     }
 
+    ((*hptilde)->data->data)[j] = hp_val;
+    ((*hctilde)->data->data)[j] = hc_val;
 
-    ((*hptilde)->data->data)[i] = hp_val;
-    ((*hctilde)->data->data)[i] = hc_val;
+    phis[i] = phasing;
 
     skip: /* this statement intentionally left blank */;
   }
 
+  /* Correct phasing so we coalesce at t=0 (with the definition of the epoch=-1/deltaF above) */
+  /* We apply the same time shift to hptilde and hctilde based on the overall phasing returned by PhenomPCoreOneFrequency */
+  if (deltaF<=0)
+    XLAL_PRINT_WARNING("Warning: Depending on specified frequency sequence correction to time of coalescence may not be accurate.\n");
+  /* Set up spline for phase */
+  gsl_interp_accel *acc = gsl_interp_accel_alloc();
+  gsl_spline *phiI = gsl_spline_alloc(gsl_interp_cspline, L_fCut);
+
+  gsl_spline_init(phiI, freqs->data, phis, L_fCut);
+
+  REAL8 f_final = PCparams->fRingDown; // This isn't quite the same as the SEOBNRv2 ringdown frequency
+  XLAL_PRINT_INFO("f_ringdown = %g\n", f_final);
+
+  // Prevent gsl interpolation errors
+  if (f_final > freqs->data[L_fCut-1])
+    f_final = freqs->data[L_fCut-1];
+  if (f_final < freqs->data[0])
+    XLAL_ERROR(XLAL_EDOM, "f_ringdown < f_min\n");
+
+  /* Time correction is t(f_final) = 1/(2pi) dphi/df (f_final) */
+  REAL8 t_corr = gsl_spline_eval_deriv(phiI, f_final, acc) / (2*LAL_PI);
+  XLAL_PRINT_INFO("t_corr = %g\n", t_corr);
+
+  /* Now correct phase */
+  for (UINT4 i=0; i<L_fCut; i++) { // loop over frequency points in sequence
+    double f = freqs->data[i];
+    int j = i + offset; // shift index for frequency series if needed
+    ((*hptilde)->data->data)[j] *= cexp(-2*LAL_PI * I * f * t_corr);
+    ((*hctilde)->data->data)[j] *= cexp(-2*LAL_PI * I * f * t_corr);
+  }
+
+  gsl_spline_free(phiI);
+  gsl_interp_accel_free(acc);
+  XLALFree(phis);
+
+  XLALFree(PCparams);
+  XLALDestroyREAL8Sequence(freqs);
+
   if( errcode != XLAL_SUCCESS )
     XLAL_ERROR(errcode);
   else
@@ -420,7 +598,7 @@ int XLALSimIMRPhenomP(
 /******************************* PhenomP internal functions *********************************/
 
 /* Internal core function to calculate PhenomP polarizations for a single frequency. */
-int PhenomPCore(
+int PhenomPCoreOneFrequency(
   const REAL8 fHz,                        /**< Frequency (Hz) */
   const REAL8 eta,                        /**< Symmetric mass ratio */
   const REAL8 chi_eff,                    /**< Dimensionless effective total aligned spin */
@@ -429,12 +607,13 @@ int PhenomPCore(
   const REAL8 M,                          /**< Total mass (Solar masses) */
   const REAL8 phic,                       /**< Orbital phase at the peak of the underlying non precessing model (rad) */
   BBHPhenomCParams *PCparams,             /**< Internal PhenomC parameters */
-  NNLOanglecoeffs *angcoeffs,  		  /**< Struct with PN coeffs for the NNLO angles */
+  NNLOanglecoeffs *angcoeffs,  		        /**< Struct with PN coeffs for the NNLO angles */
   SpinWeightedSphericalHarmonic_l2 *Y2m,  /**< Struct of l=2 spherical harmonics of spin weight -2 */
-  const REAL8 alphaoffset,                /**< f_ref dependent offset for alpha angle */
+  const REAL8 alphaoffset,                /**< f_ref dependent offset for alpha angle (azimuthal precession angle) */
   const REAL8 epsilonoffset,              /**< f_ref dependent offset for epsilon angle */
   COMPLEX16 *hp,                          /**< Output: \tilde h_+ */
-  COMPLEX16 *hc)                          /**< Output: \tilde h_+ */
+  COMPLEX16 *hc,                          /**< Output: \tilde h_+ */
+  REAL8 *phasing)                         /**< Output: overall phasing */
 {
 
   /* Calculate PhenomC amplitude and phase for a given frequency. */
@@ -532,6 +711,9 @@ int PhenomPCore(
   *hp = eps_phase_hPC * hp_sum;
   *hc = eps_phase_hPC * hc_sum;
 
+  //*phasing = -phPhenomC -2*epsilon; // ignore alpha contributions
+  *phasing = -phPhenomC; // ignore alpha and epsilon contributions
+
   return XLAL_SUCCESS;
 }
 
@@ -761,8 +943,10 @@ BBHPhenomCParams *ComputeIMRPhenomCParamsRDmod(
   const REAL8 eta = m1 * m2 / (M * M);
 
   REAL8 finspin = FinalSpinBarausse2009_all_spin_on_larger_BH(eta, chi, chip);
-  if( fabs(finspin) > 1.0 )
+  if( fabs(finspin) > 1.0 ) {
+    XLAL_PRINT_ERROR("Error: magnitude of final spin > 1!");
     XLAL_ERROR_NULL( XLAL_EDOM );
+  }
 
   p->afin = finspin;
 
