@@ -288,7 +288,6 @@ XLALFindSphericalOrbit( const gsl_vector *x, /**<< Parameters requested by gsl r
                         gsl_vector *f        /**<< Function values for the given parameters */
                       )
 {
-
   SEOBRootParams *rootParams = (SEOBRootParams *) params;
 
   REAL8 py, pz, r, ptheta, pphi;
@@ -303,7 +302,7 @@ XLALFindSphericalOrbit( const gsl_vector *x, /**<< Parameters requested by gsl r
   rootParams->values[4] = py = gsl_vector_get( x, 1 );
   rootParams->values[5] = pz = gsl_vector_get( x, 2 );
 
-  //printf( "Values r = %.16e, py = %.16e, pz = %.16e\n", r, py, pz );
+ // printf( "Values r = %.16e, py = %.16e, pz = %.16e\n", r, py, pz );
 
   ptheta = - r * pz;
   pphi   = r * py;
@@ -343,7 +342,7 @@ XLALFindSphericalOrbit( const gsl_vector *x, /**<< Parameters requested by gsl r
   gsl_vector_set( f, 2, dHdpphi - rootParams->omega );
 
   //printf( "Current funcvals = %.16e %.16e %.16e\n", gsl_vector_get( f, 0 ), gsl_vector_get( f, 1 ),
-  //   /*gsl_vector_get( f, 2 )*/dHdpphi );
+   //  gsl_vector_get( f, 2 )/*dHdpphi*/ );
 
   return XLAL_SUCCESS;
 }
@@ -523,6 +522,8 @@ static int XLALSimIMRSpinEOBInitialConditions(
   /* Variable to keep track of whether the user requested the tortoise */
   int tmpTortoise;
 
+  UINT4 SpinAlignedEOBversion;
+
   REAL8 mTotal;
   REAL8 eta;
   REAL8 omega, v0;   /* Initial velocity and angular frequency */
@@ -544,13 +545,15 @@ static int XLALSimIMRSpinEOBInitialConditions(
   /* We will use temporary vectors to do this */
   REAL8 tmpS1[3];
   REAL8 tmpS2[3];
+  REAL8 tmpS1Norm[3];
+  REAL8 tmpS2Norm[3];
 
   REAL8Vector qCartVec, pCartVec;
-  REAL8Vector s1Vec, s2Vec;
+  REAL8Vector s1Vec, s2Vec, s1VecNorm, s2VecNorm;
   REAL8Vector sKerr, sStar;
   REAL8       sKerrData[3], sStarData[3];
-  REAL8       a = 0., chiS, chiA;
-  REAL8       chi1, chi2;
+  REAL8       a = 0.; //, chiS, chiA;
+  //REAL8       chi1, chi2;
 
   /* We will need a full values vector for calculating derivs of Hamiltonian */
   REAL8 sphValues[12];
@@ -576,16 +579,25 @@ static int XLALSimIMRSpinEOBInitialConditions(
 
   memset( &rootParams, 0, sizeof( rootParams ) );
 
-  memcpy( tmpS1, spin1, sizeof(tmpS1) );
-  memcpy( tmpS2, spin2, sizeof(tmpS2) );
-
   mTotal = mass1 + mass2;
   eta    = mass1 * mass2 / (mTotal * mTotal);
-
+  memcpy( tmpS1, spin1, sizeof(tmpS1) );
+  memcpy( tmpS2, spin2, sizeof(tmpS2) );
+  memcpy( tmpS1Norm, spin1, sizeof(tmpS1Norm) );
+  memcpy( tmpS2Norm, spin2, sizeof(tmpS2Norm) );
+  for ( i = 0; i < 3; i++ )
+  {
+     tmpS1Norm[i] /= mTotal * mTotal;
+     tmpS2Norm[i] /= mTotal * mTotal;
+  }
+  SpinAlignedEOBversion = params->seobCoeffs->SpinAlignedEOBversion;
   /* We compute the ICs for the non-tortoise p, and convert at the end */
   tmpTortoise      = params->tortoise;
   params->tortoise = 0;
 
+  EOBNonQCCoeffs *nqcCoeffs = NULL;
+  nqcCoeffs = params->nqcCoeffs;
+
   /* STEP 1) Rotate to LNhat0 along z-axis and N0 along x-axis frame, where LNhat0 and N0 are initial normal to 
    *         orbital plane and initial orbital separation;
    */
@@ -651,6 +663,8 @@ static int XLALSimIMRSpinEOBInitialConditions(
   ApplyRotationMatrix( rotMatrix, LnHat );
   ApplyRotationMatrix( rotMatrix, tmpS1 );
   ApplyRotationMatrix( rotMatrix, tmpS2 );
+  ApplyRotationMatrix( rotMatrix, tmpS1Norm );
+  ApplyRotationMatrix( rotMatrix, tmpS2Norm );
 
   /* XXX Test code XXX */
   /*printf( "\nAfter applying rotation matrix:\n\n" );
@@ -793,6 +807,8 @@ static int XLALSimIMRSpinEOBInitialConditions(
   ApplyRotationMatrix( rotMatrix2, LnHat );
   ApplyRotationMatrix( rotMatrix2, tmpS1 );
   ApplyRotationMatrix( rotMatrix2, tmpS2 );
+  ApplyRotationMatrix( rotMatrix2, tmpS1Norm );
+  ApplyRotationMatrix( rotMatrix2, tmpS2Norm );
   ApplyRotationMatrix( rotMatrix2, qCart );
   ApplyRotationMatrix( rotMatrix2, pCart );
 
@@ -829,9 +845,11 @@ static int XLALSimIMRSpinEOBInitialConditions(
   if ( d2Hdr2 != 0.0 )
   {
     /* We will need to calculate the Hamiltonian to get the flux */
-    s1Vec.length = s2Vec.length = sKerr.length = sStar.length = 3;
+    s1Vec.length = s2Vec.length = s1VecNorm.length = s2VecNorm.length = sKerr.length = sStar.length = 3;
     s1Vec.data = tmpS1;
     s2Vec.data = tmpS2;
+    s1VecNorm.data = tmpS1Norm;
+    s2VecNorm.data = tmpS2Norm;
     sKerr.data = sKerrData;
     sStar.data = sStarData;
 
@@ -839,23 +857,22 @@ static int XLALSimIMRSpinEOBInitialConditions(
     qCartVec.data   = qCart;
     pCartVec.data   = pCart;
 
-    chi1 = tmpS1[0]*LnHat[0] + tmpS1[1]*LnHat[1] + tmpS1[2]*LnHat[2];
-    chi2 = tmpS2[0]*LnHat[0] + tmpS2[1]*LnHat[1] + tmpS2[2]*LnHat[2];
+    //chi1 = tmpS1[0]*LnHat[0] + tmpS1[1]*LnHat[1] + tmpS1[2]*LnHat[2];
+    //chi2 = tmpS2[0]*LnHat[0] + tmpS2[1]*LnHat[1] + tmpS2[2]*LnHat[2];
 
     //printf( "magS1 = %.16e, magS2 = %.16e\n", chi1, chi2 );
 
-    chiS = 0.5 * ( chi1 / (mass1*mass1) + chi2 / (mass2*mass2) );
-    chiA = 0.5 * ( chi1 / (mass1*mass1) - chi2 / (mass2*mass2) );
+    //chiS = 0.5 * ( chi1 / (mass1*mass1) + chi2 / (mass2*mass2) );
+    //chiA = 0.5 * ( chi1 / (mass1*mass1) - chi2 / (mass2*mass2) );
 
     XLALSimIMRSpinEOBCalculateSigmaKerr( &sKerr, mass1, mass2, &s1Vec, &s2Vec );
     XLALSimIMRSpinEOBCalculateSigmaStar( &sStar, mass1, mass2, &s1Vec, &s2Vec );
 
     /* The a in the flux has been set to zero, but not in the Hamiltonian */
     a = sqrt(sKerr.data[0]*sKerr.data[0] + sKerr.data[1]*sKerr.data[1] + sKerr.data[2]*sKerr.data[2]);
-    XLALSimIMREOBCalcSpinFacWaveformCoefficients( params->eobParams->hCoeffs, mass1, mass2, eta, /*a*/0.0, chiS, chiA );
-    XLALSimIMRCalculateSpinEOBHCoeffs( params->seobCoeffs, eta, a );
-
-    ham = XLALSimIMRSpinEOBHamiltonian( eta, &qCartVec, &pCartVec, &sKerr, &sStar, params->tortoise, params->seobCoeffs );
+    //XLALSimIMREOBCalcSpinFacWaveformCoefficients( params->eobParams->hCoeffs, mass1, mass2, eta, /*a*/0.0, chiS, chiA );
+    //XLALSimIMRCalculateSpinEOBHCoeffs( params->seobCoeffs, eta, a );
+    ham = XLALSimIMRSpinEOBHamiltonian( eta, &qCartVec, &pCartVec, &s1VecNorm, &s2VecNorm, &sKerr, &sStar, params->tortoise, params->seobCoeffs );
 
     //printf( "hamiltonian at this point is %.16e\n", ham );
 
@@ -871,7 +888,7 @@ static int XLALSimIMRSpinEOBInitialConditions(
     polarData[2] = pSph[0];
     polarData[3] = pSph[2];
 
-    flux  = XLALInspiralSpinFactorizedFlux( &polarDynamics, omega, params, ham, lMax );
+    flux  = XLALInspiralSpinFactorizedFlux( &polarDynamics, nqcCoeffs, omega, params, ham, lMax, SpinAlignedEOBversion );
     flux  = flux / eta;
 
     rDot  = - flux / dEdr;
@@ -908,6 +925,8 @@ static int XLALSimIMRSpinEOBInitialConditions(
   ApplyRotationMatrix( invMatrix2, LnHat );
   ApplyRotationMatrix( invMatrix2, tmpS1 );
   ApplyRotationMatrix( invMatrix2, tmpS2 );
+  ApplyRotationMatrix( invMatrix2, tmpS1Norm );
+  ApplyRotationMatrix( invMatrix2, tmpS2Norm );
   ApplyRotationMatrix( invMatrix2, qCart );
   ApplyRotationMatrix( invMatrix2, pCart );
 
@@ -917,6 +936,8 @@ static int XLALSimIMRSpinEOBInitialConditions(
   ApplyRotationMatrix( invMatrix, LnHat );
   ApplyRotationMatrix( invMatrix, tmpS1 );
   ApplyRotationMatrix( invMatrix, tmpS2 );
+  ApplyRotationMatrix( invMatrix, tmpS1Norm );
+  ApplyRotationMatrix( invMatrix, tmpS2Norm );
   ApplyRotationMatrix( invMatrix, qCart );
   ApplyRotationMatrix( invMatrix, pCart );
 
@@ -943,8 +964,8 @@ static int XLALSimIMRSpinEOBInitialConditions(
   /* Now copy the initial conditions back to the return vector */
   memcpy( initConds->data, qCart, sizeof(qCart) );
   memcpy( initConds->data+3, pCart, sizeof(pCart) );
-  memcpy( initConds->data+6, tmpS1, sizeof(tmpS1) );
-  memcpy( initConds->data+9, tmpS2, sizeof(tmpS2) );
+  memcpy( initConds->data+6, tmpS1Norm, sizeof(tmpS1Norm) );
+  memcpy( initConds->data+9, tmpS2Norm, sizeof(tmpS2Norm) );
 
   //printf( "THE FINAL INITIAL CONDITIONS:\n");
   /*printf( " %.16e %.16e %.16e\n%.16e %.16e %.16e\n%.16e %.16e %.16e\n%.16e %.16e %.16e\n", initConds->data[0], initConds->data[1], initConds->data[2],
