@@ -28,12 +28,18 @@
 #include <lal/Date.h>
 #include <lal/FrequencySeries.h>
 #include <lal/LALConstants.h>
+#include <lal/Sequence.h>
 #include <lal/LALDatatypes.h>
 #include <lal/LALSimInspiral.h>
 #include <lal/Units.h>
 #include <lal/XLALError.h>
 #include "LALSimInspiralPNCoefficients.c"
 
+#ifndef _OPENMP
+#define omp ignore
+#endif
+
+
 /* This function allows SWIG to wrap the TaylorF2 phasing coefficients
  * for use in external Python code
  */
@@ -63,6 +69,278 @@ int XLALSimInspiralTaylorF2AlignedPhasing(
     return XLAL_SUCCESS;
 }
 
+int XLALSimInspiralTaylorF2Core(
+        COMPLEX16FrequencySeries **htilde_out, /**< FD waveform */
+	const REAL8Sequence *freqs,            /**< frequency points at which to evaluate the waveform (Hz) */
+        const REAL8 phi_ref,                   /**< reference orbital phase (rad) */
+        const REAL8 m1_SI,                     /**< mass of companion 1 (kg) */
+        const REAL8 m2_SI,                     /**< mass of companion 2 (kg) */
+        const REAL8 S1z,                       /**<  z component of the spin of companion 1 */
+        const REAL8 S2z,                       /**<  z component of the spin of companion 2  */
+        const REAL8 f_ref,                     /**< Reference GW frequency (Hz) - if 0 reference point is coalescence */
+	const REAL8 shft,		       /**< time shift to be applied to frequency-domain phase (sec)*/
+        const REAL8 r,                         /**< distance of source (m) */
+        const REAL8 quadparam1,                /**< quadrupole deformation parameter of body 1 (dimensionless, 1 for BH) */
+        const REAL8 quadparam2,                /**< quadrupole deformation parameter of body 2 (dimensionless, 1 for BH) */
+        const REAL8 lambda1,                   /**< (tidal deformation of body 1)/(mass of body 1)^5 */
+        const REAL8 lambda2,                   /**< (tidal deformation of body 2)/(mass of body 2)^5 */
+        const LALSimInspiralSpinOrder spinO,  /**< twice PN order of spin effects */
+        const LALSimInspiralTidalOrder tideO,  /**< flag to control tidal effects */
+        const INT4 phaseO,                     /**< twice PN phase order */
+        const INT4 amplitudeO                  /**< twice PN amplitude order */
+        )
+{
+
+    if (!htilde_out) XLAL_ERROR(XLAL_EFAULT);
+    if (!freqs) XLAL_ERROR(XLAL_EFAULT);
+    /* external: SI; internal: solar masses */
+    const REAL8 m1 = m1_SI / LAL_MSUN_SI;
+    const REAL8 m2 = m2_SI / LAL_MSUN_SI;
+    const REAL8 m = m1 + m2;
+    const REAL8 m_sec = m * LAL_MTSUN_SI;  /* total mass in seconds */
+    const REAL8 eta = m1 * m2 / (m * m);
+    const REAL8 piM = LAL_PI * m_sec;
+    const REAL8 m1OverM = m1 / m;
+    const REAL8 m2OverM = m2 / m;
+    REAL8 amp0;
+    size_t i;
+    COMPLEX16 *data = NULL;
+    LIGOTimeGPS tC = {0, 0};
+    INT4 iStart = 0;
+
+    COMPLEX16FrequencySeries *htilde = NULL;
+
+    if (*htilde_out) { //case when htilde_out has been allocated in XLALSimInspiralTaylorF2
+	    htilde = *htilde_out;
+	    iStart = htilde->data->length - freqs->length; //index shift to fill pre-allocated data
+	    if(iStart < 0) XLAL_ERROR(XLAL_EFAULT);
+    }
+    else { //otherwise allocate memory here
+	    htilde = XLALCreateCOMPLEX16FrequencySeries("htilde: FD waveform", &tC, freqs->data[0], 0., &lalStrainUnit, freqs->length);
+	    if (!htilde) XLAL_ERROR(XLAL_EFUNC);
+	    XLALUnitDivide(&htilde->sampleUnits, &htilde->sampleUnits, &lalSecondUnit);
+    }
+
+    /* phasing coefficients */
+    PNPhasingSeries pfa;
+    XLALSimInspiralPNPhasing_F2(&pfa, m1, m2, S1z, S2z, S1z*S1z, S2z*S2z, S1z*S2z, quadparam1, quadparam2, spinO);
+
+    REAL8 pfaN = 0.;
+    REAL8 pfa2 = 0.; REAL8 pfa3 = 0.; REAL8 pfa4 = 0.;
+    REAL8 pfa5 = 0.; REAL8 pfl5 = 0.;
+    REAL8 pfa6 = 0.; REAL8 pfl6 = 0.;
+    REAL8 pfa7 = 0.;
+
+    switch (phaseO)
+    {
+        case -1:
+        case 7:
+            pfa7 = pfa.v[7];
+        case 6:
+            pfa6 = pfa.v[6];
+            pfl6 = pfa.vlogv[6];
+        case 5:
+            pfa5 = pfa.v[5];
+            pfl5 = pfa.vlogv[5];
+        case 4:
+            pfa4 = pfa.v[4];
+        case 3:
+            pfa3 = pfa.v[3];
+        case 2:
+            pfa2 = pfa.v[2];
+        case 0:
+            pfaN = pfa.v[0];
+            break;
+	case 1:
+	    XLALPrintWarning( "There is no 0.5PN phase coefficient, returning Newtonian-order phase.\n" );
+            pfaN = pfa.v[0];
+            break;
+        default:
+            XLAL_ERROR(XLAL_ETYPE, "Invalid phase PN order %d", phaseO);
+    }
+
+    /* Validate expansion order arguments.
+     * This must be done here instead of in the OpenMP parallel loop
+     * because when OpenMP parallelization is turned on, early exits
+     * from loops (via return or break statements) are not permitted.
+     */
+
+    /* Validate amplitude PN order. */
+    switch (amplitudeO)
+    {
+        case -1:
+        case 7:
+        case 6:
+        case 5:
+        case 4:
+        case 3:
+        case 2:
+        case 0:
+            break;
+        default:
+            XLAL_ERROR(XLAL_ETYPE, "Invalid amplitude PN order %d", amplitudeO);
+    }
+
+    /* Generate tidal terms separately.
+     * Enums specifying tidal order are in LALSimInspiralWaveformFlags.h
+     */
+    REAL8 pft10 = 0.;
+    REAL8 pft12 = 0.;
+    switch( tideO )
+    {
+	    case LAL_SIM_INSPIRAL_TIDAL_ORDER_ALL:
+        case LAL_SIM_INSPIRAL_TIDAL_ORDER_6PN:
+	    pft12 = pfaN * (lambda1*XLALSimInspiralTaylorF2Phasing_12PNTidalCoeff(m1OverM) + lambda2*XLALSimInspiralTaylorF2Phasing_12PNTidalCoeff(m2OverM) );
+        case LAL_SIM_INSPIRAL_TIDAL_ORDER_5PN:
+            pft10 = pfaN * ( lambda1*XLALSimInspiralTaylorF2Phasing_10PNTidalCoeff(m1OverM) + lambda2*XLALSimInspiralTaylorF2Phasing_10PNTidalCoeff(m2OverM) );
+        case LAL_SIM_INSPIRAL_TIDAL_ORDER_0PN:
+            break;
+        default:
+            XLAL_ERROR(XLAL_EINVAL, "Invalid tidal PN order %d", tideO);
+    }
+
+    /* The flux and energy coefficients below are used to compute SPA amplitude corrections */
+
+    /* flux coefficients */
+    const REAL8 FTaN = XLALSimInspiralPNFlux_0PNCoeff(eta);
+    const REAL8 FTa2 = XLALSimInspiralPNFlux_2PNCoeff(eta);
+    const REAL8 FTa3 = XLALSimInspiralPNFlux_3PNCoeff(eta);
+    const REAL8 FTa4 = XLALSimInspiralPNFlux_4PNCoeff(eta);
+    const REAL8 FTa5 = XLALSimInspiralPNFlux_5PNCoeff(eta);
+    const REAL8 FTl6 = XLALSimInspiralPNFlux_6PNLogCoeff(eta);
+    const REAL8 FTa6 = XLALSimInspiralPNFlux_6PNCoeff(eta);
+    const REAL8 FTa7 = XLALSimInspiralPNFlux_7PNCoeff(eta);
+
+    /* energy coefficients */
+    const REAL8 dETaN = 2. * XLALSimInspiralPNEnergy_0PNCoeff(eta);
+    const REAL8 dETa1 = 2. * XLALSimInspiralPNEnergy_2PNCoeff(eta);
+    const REAL8 dETa2 = 3. * XLALSimInspiralPNEnergy_4PNCoeff(eta);
+    const REAL8 dETa3 = 4. * XLALSimInspiralPNEnergy_6PNCoeff(eta);
+
+
+    /* Perform some initial checks */
+    if (m1_SI <= 0) XLAL_ERROR(XLAL_EDOM);
+    if (m2_SI <= 0) XLAL_ERROR(XLAL_EDOM);
+    if (f_ref < 0) XLAL_ERROR(XLAL_EDOM);
+    if (r <= 0) XLAL_ERROR(XLAL_EDOM);
+
+    /* extrinsic parameters */
+    amp0 = -4. * m1 * m2 / r * LAL_MRSUN_SI * LAL_MTSUN_SI * sqrt(LAL_PI/12.L);
+
+    data = htilde->data->data;
+
+    /* Compute the SPA phase at the reference point
+     * N.B. f_ref == 0 means we define the reference time/phase at "coalescence"
+     * when the frequency approaches infinity. In that case,
+     * the integrals Eq. 3.15 of arXiv:0907.0700 vanish when evaluated at
+     * f_ref == infinity. If f_ref is finite, we must compute the SPA phase
+     * evaluated at f_ref, store it as ref_phasing and subtract it off.
+     */
+    REAL8 ref_phasing = 0.;
+    if( f_ref != 0. ) {
+        const REAL8 vref = cbrt(piM*f_ref);
+        const REAL8 logvref = log(vref);
+        const REAL8 v2ref = vref * vref;
+        const REAL8 v3ref = vref * v2ref;
+        const REAL8 v4ref = vref * v3ref;
+        const REAL8 v5ref = vref * v4ref;
+        const REAL8 v6ref = vref * v5ref;
+        const REAL8 v7ref = vref * v6ref;
+        const REAL8 v8ref = vref * v7ref;
+        const REAL8 v9ref = vref * v8ref;
+        const REAL8 v10ref = vref * v9ref;
+        const REAL8 v12ref = v2ref * v10ref;
+        ref_phasing += pfa7 * v7ref;
+        ref_phasing += (pfa6 + pfl6 * logvref) * v6ref;
+        ref_phasing += (pfa5 + pfl5 * logvref) * v5ref;
+        ref_phasing += pfa4 * v4ref;
+        ref_phasing += pfa3 * v3ref;
+        ref_phasing += pfa2 * v2ref;
+        ref_phasing += pfaN;
+
+        /* Tidal terms in reference phasing */
+        ref_phasing += pft12 * v12ref;
+        ref_phasing += pft10 * v10ref;
+
+        ref_phasing /= v5ref;
+    } /* End of if(f_ref != 0) block */
+
+    #pragma omp parallel for
+    for (i = 0; i < freqs->length; i++) {
+        const REAL8 f = freqs->data[i];
+        const REAL8 v = cbrt(piM*f);
+        const REAL8 logv = log(v);
+        const REAL8 v2 = v * v;
+        const REAL8 v3 = v * v2;
+        const REAL8 v4 = v * v3;
+        const REAL8 v5 = v * v4;
+        const REAL8 v6 = v * v5;
+        const REAL8 v7 = v * v6;
+        const REAL8 v8 = v * v7;
+        const REAL8 v9 = v * v8;
+        const REAL8 v10 = v * v9;
+        const REAL8 v12 = v2 * v10;
+        REAL8 phasing = 0.;
+        REAL8 dEnergy = 0.;
+        REAL8 flux = 0.;
+        REAL8 amp;
+
+        phasing += pfa7 * v7;
+        phasing += (pfa6 + pfl6 * logv) * v6;
+        phasing += (pfa5 + pfl5 * logv) * v5;
+        phasing += pfa4 * v4;
+        phasing += pfa3 * v3;
+        phasing += pfa2 * v2;
+        phasing += pfaN;
+
+        /* Tidal terms in phasing */
+        phasing += pft12 * v12;
+        phasing += pft10 * v10;
+
+    /* WARNING! Amplitude orders beyond 0 have NOT been reviewed!
+     * Use at your own risk. The default is to turn them off.
+     * These do not currently include spin corrections.
+     * Note that these are not higher PN corrections to the amplitude.
+     * They are the corrections to the leading-order amplitude arising
+     * from the stationary phase approximation. See for instance
+     * Eq 6.9 of arXiv:0810.5336
+     */
+	switch (amplitudeO)
+        {
+            case 7:
+                flux += FTa7 * v7;
+            case 6:
+                flux += (FTa6 + FTl6*logv) * v6;
+                dEnergy += dETa3 * v6;
+            case 5:
+                flux += FTa5 * v5;
+            case 4:
+                flux += FTa4 * v4;
+                dEnergy += dETa2 * v4;
+            case 3:
+                flux += FTa3 * v3;
+            case 2:
+                flux += FTa2 * v2;
+                dEnergy += dETa1 * v2;
+            case -1: /* Default to no SPA amplitude corrections */
+            case 0:
+                flux += 1.;
+                dEnergy += 1.;
+        }
+
+        phasing /= v5;
+        flux *= FTaN * v10;
+        dEnergy *= dETaN * v;
+        // Note the factor of 2 b/c phi_ref is orbital phase
+        phasing += shft * f - 2.*phi_ref - ref_phasing;
+        amp = amp0 * sqrt(-dEnergy/flux) * v;
+        data[i+iStart] = amp * cos(phasing - LAL_PI_4)
+                - amp * sin(phasing - LAL_PI_4) * 1.0j;
+    }
+
+    *htilde_out = htilde;
+    return XLAL_SUCCESS;
+}
 
 /**
  * Computes the stationary phase approximation to the Fourier transform of
@@ -108,115 +386,20 @@ int XLALSimInspiralTaylorF2(
     const REAL8 m2 = m2_SI / LAL_MSUN_SI;
     const REAL8 m = m1 + m2;
     const REAL8 m_sec = m * LAL_MTSUN_SI;  /* total mass in seconds */
-    const REAL8 eta = m1 * m2 / (m * m);
+    // const REAL8 eta = m1 * m2 / (m * m);
     const REAL8 piM = LAL_PI * m_sec;
     const REAL8 vISCO = 1. / sqrt(6.);
     const REAL8 fISCO = vISCO * vISCO * vISCO / piM;
-    const REAL8 m1OverM = m1 / m;
-    const REAL8 m2OverM = m2 / m;
-    REAL8 shft, amp0, f_max;
-    size_t i, n, iStart;
-    COMPLEX16 *data = NULL;
+    //const REAL8 m1OverM = m1 / m;
+    // const REAL8 m2OverM = m2 / m;
+    REAL8 shft, f_max;
+    size_t i, n;
+    INT4 iStart;
+    REAL8Sequence *freqs = NULL;
     LIGOTimeGPS tC = {0, 0};
+    int ret;
 
-    /* phasing coefficients */
-    PNPhasingSeries pfa;
-    XLALSimInspiralPNPhasing_F2(&pfa, m1, m2, S1z, S2z, S1z*S1z, S2z*S2z, S1z*S2z, quadparam1, quadparam2, spinO);
-
-    REAL8 pfaN = 0.;
-    REAL8 pfa2 = 0.; REAL8 pfa3 = 0.; REAL8 pfa4 = 0.;
-    REAL8 pfa5 = 0.; REAL8 pfl5 = 0.;
-    REAL8 pfa6 = 0.; REAL8 pfl6 = 0.;
-    REAL8 pfa7 = 0.;
-
-    switch (phaseO)
-    {
-        case -1:
-        case 7:
-            pfa7 = pfa.v[7];
-        case 6:
-            pfa6 = pfa.v[6];
-            pfl6 = pfa.vlogv[6];
-        case 5:
-            pfa5 = pfa.v[5];
-            pfl5 = pfa.vlogv[5];
-        case 4:
-            pfa4 = pfa.v[4];
-        case 3:
-            pfa3 = pfa.v[3];
-        case 2:
-            pfa2 = pfa.v[2];
-	case 1:
-	    XLALPrintWarning( "There is no 0.5PN phase coefficient, returning Newtonian-order phase.\n" );
-        case 0:
-            pfaN = pfa.v[0];
-            break;
-        default:
-            XLAL_ERROR(XLAL_ETYPE, "Invalid phase PN order %s", phaseO);
-    }
-
-    /* Validate expansion order arguments.
-     * This must be done here instead of in the OpenMP parallel loop
-     * because when OpenMP parallelization is turned on, early exits
-     * from loops (via return or break statements) are not permitted.
-     */
-
-    /* Validate amplitude PN order. */
-    switch (amplitudeO)
-    {
-        case -1:
-        case 7:
-        case 6:
-        case 5:
-        case 4:
-        case 3:
-        case 2:
-        case 0:
-            break;
-        default:
-            XLAL_ERROR(XLAL_ETYPE, "Invalid amplitude PN order %s", amplitudeO);
-    }
-
-    /* Generate tidal terms separately.
-     * Enums specifying tidal order are in LALSimInspiralWaveformFlags.h
-     */
-    REAL8 pft10 = 0.;
-    REAL8 pft12 = 0.;
-    switch( tideO )
-    {
-	    case LAL_SIM_INSPIRAL_TIDAL_ORDER_ALL:
-        case LAL_SIM_INSPIRAL_TIDAL_ORDER_6PN:
-            pft12 = pfaN * ( XLALSimInspiralTaylorF2Phasing_12PNTidalCoeff(m1OverM, lambda1)
-                            + XLALSimInspiralTaylorF2Phasing_12PNTidalCoeff(m2OverM, lambda2) );
-        case LAL_SIM_INSPIRAL_TIDAL_ORDER_5PN:
-            pft10 = pfaN * ( XLALSimInspiralTaylorF2Phasing_10PNTidalCoeff(m1OverM, lambda1)
-                            + XLALSimInspiralTaylorF2Phasing_10PNTidalCoeff(m2OverM, lambda2) );
-        case LAL_SIM_INSPIRAL_TIDAL_ORDER_0PN:
-            break;
-        default:
-            XLAL_ERROR(XLAL_EINVAL, "Invalid tidal PN order %s", tideO);
-    }
-
-    /* The flux and energy coefficients below are used to compute SPA amplitude corrections */
-
-    /* flux coefficients */
-    const REAL8 FTaN = XLALSimInspiralPNFlux_0PNCoeff(eta);
-    const REAL8 FTa2 = XLALSimInspiralPNFlux_2PNCoeff(eta);
-    const REAL8 FTa3 = XLALSimInspiralPNFlux_3PNCoeff(eta);
-    const REAL8 FTa4 = XLALSimInspiralPNFlux_4PNCoeff(eta);
-    const REAL8 FTa5 = XLALSimInspiralPNFlux_5PNCoeff(eta);
-    const REAL8 FTl6 = XLALSimInspiralPNFlux_6PNLogCoeff(eta);
-    const REAL8 FTa6 = XLALSimInspiralPNFlux_6PNCoeff(eta);
-    const REAL8 FTa7 = XLALSimInspiralPNFlux_7PNCoeff(eta);
-
-    /* energy coefficients */
-    const REAL8 dETaN = 2. * XLALSimInspiralPNEnergy_0PNCoeff(eta);
-    const REAL8 dETa1 = 2. * XLALSimInspiralPNEnergy_2PNCoeff(eta);
-    const REAL8 dETa2 = 3. * XLALSimInspiralPNEnergy_4PNCoeff(eta);
-    const REAL8 dETa3 = 4. * XLALSimInspiralPNEnergy_6PNCoeff(eta);
-
-
-    COMPLEX16FrequencySeries *htilde;
+    COMPLEX16FrequencySeries *htilde = NULL;
 
     /* Perform some initial checks */
     if (!htilde_out) XLAL_ERROR(XLAL_EFAULT);
@@ -241,125 +424,26 @@ int XLALSimInspiralTaylorF2(
     memset(htilde->data->data, 0, n * sizeof(COMPLEX16));
     XLALUnitDivide(&htilde->sampleUnits, &htilde->sampleUnits, &lalSecondUnit);
 
-    /* extrinsic parameters */
-    amp0 = -4. * m1 * m2 / r * LAL_MRSUN_SI * LAL_MTSUN_SI * sqrt(LAL_PI/12.L);
-    shft = LAL_TWOPI * (tC.gpsSeconds + 1e-9 * tC.gpsNanoSeconds);
-
-    const REAL8 log4=log(4.0);
-
     /* Fill with non-zero vals from fStart to f_max */
-    iStart = (size_t) ceil(fStart / deltaF);
-    data = htilde->data->data;
+    iStart = (INT4) ceil(fStart / deltaF);
 
-    /* Compute the SPA phase at the reference point
-     * N.B. f_ref == 0 means we define the reference time/phase at "coalescence"
-     * when the frequency approaches infinity. In that case,
-     * the integrals Eq. 3.15 of arXiv:0907.0700 vanish when evaluated at
-     * f_ref == infinity. If f_ref is finite, we must compute the SPA phase
-     * evaluated at f_ref, store it as ref_phasing and subtract it off.
-     */
-    REAL8 ref_phasing = 0.;
-    if( f_ref != 0. ) {
-        const REAL8 vref = cbrt(piM*f_ref);
-        const REAL8 logvref = log(vref);
-        const REAL8 v2ref = vref * vref;
-        const REAL8 v3ref = vref * v2ref;
-        const REAL8 v4ref = vref * v3ref;
-        const REAL8 v5ref = vref * v4ref;
-        const REAL8 v6ref = vref * v5ref;
-        const REAL8 v7ref = vref * v6ref;
-        const REAL8 v8ref = vref * v7ref;
-        const REAL8 v9ref = vref * v8ref;
-        const REAL8 v10ref = vref * v9ref;
-        const REAL8 v12ref = v2ref * v10ref;
-        ref_phasing += pfa7 * v7ref;
-        ref_phasing += (pfa6 + pfl6 * logvref) * v6ref;
-        ref_phasing += (pfa5 + pfl5 * logvref) * v5ref;
-        ref_phasing += pfa4 * v4ref;
-        ref_phasing += pfa3 * v3ref;
-        ref_phasing += pfa2 * v2ref;
-        ref_phasing += pfaN;
+    /* Sequence of frequencies where waveform model is to be evaluated */
+    freqs = XLALCreateREAL8Sequence(n - iStart);
 
-        /* Tidal terms in reference phasing */
-        ref_phasing += pft12 * v12ref;
-        ref_phasing += pft10 * v10ref;
-
-        ref_phasing /= v5ref;
-    } /* End of if(f_ref != 0) block */
+    /* extrinsic parameters */
+    shft = LAL_TWOPI * (tC.gpsSeconds + 1e-9 * tC.gpsNanoSeconds);
 
     #pragma omp parallel for
     for (i = iStart; i < n; i++) {
-        const REAL8 f = i * deltaF;
-        const REAL8 v = cbrt(piM*f);
-        const REAL8 logv = log(v);
-        const REAL8 v2 = v * v;
-        const REAL8 v3 = v * v2;
-        const REAL8 v4 = v * v3;
-        const REAL8 v5 = v * v4;
-        const REAL8 v6 = v * v5;
-        const REAL8 v7 = v * v6;
-        const REAL8 v8 = v * v7;
-        const REAL8 v9 = v * v8;
-        const REAL8 v10 = v * v9;
-        const REAL8 v12 = v2 * v10;
-        REAL8 phasing = 0.;
-        REAL8 dEnergy = 0.;
-        REAL8 flux = 0.;
-        REAL8 amp;
-
-        phasing += pfa7 * v7;
-        phasing += (pfa6 + pfl6 * logv) * v6;
-        phasing += (pfa5 + pfl5 * logv) * v5;
-        phasing += pfa4 * v4;
-        phasing += pfa3 * v3;
-        phasing += pfa2 * v2;
-        phasing += pfaN;
-
-        /* Tidal terms in phasing */
-        phasing += pft12 * v12;
-        phasing += pft10 * v10;
-
-    /* WARNING! Amplitude orders beyond 0 have NOT been reviewed!
-     * Use at your own risk. The default is to turn them off.
-     * These do not currently include spin corrections.
-     * Note that these are not higher PN corrections to the amplitude.
-     * They are the corrections to the leading-order amplitude arising
-     * from the stationary phase approximation. See for instance
-     * Eq 6.9 of arXiv:0810.5336
-     */
-	switch (amplitudeO)
-        {
-            case 7:
-                flux += FTa7 * v7;
-            case 6:
-                flux += (FTa6 + FTl6*2.0*(log4+logv)) * v6;
-                dEnergy += dETa3 * v6;
-            case 5:
-                flux += FTa5 * v5;
-            case 4:
-                flux += FTa4 * v4;
-                dEnergy += dETa2 * v4;
-            case 3:
-                flux += FTa3 * v3;
-            case 2:
-                flux += FTa2 * v2;
-                dEnergy += dETa1 * v2;
-            case -1: /* Default to no SPA amplitude corrections */
-            case 0:
-                flux += 1.;
-                dEnergy += 1.;
-        }
-
-        phasing /= v5;
-        flux *= FTaN * v10;
-        dEnergy *= dETaN * v;
-        // Note the factor of 2 b/c phi_ref is orbital phase
-        phasing += shft * f - 2.*phi_ref - ref_phasing;
-        amp = amp0 * sqrt(-dEnergy/flux) * v;
-        data[i] = amp * cos(phasing - LAL_PI_4)
-                - amp * sin(phasing - LAL_PI_4) * 1.0j;
+        freqs->data[i-iStart] = i * deltaF;
     }
+    ret = XLALSimInspiralTaylorF2Core(&htilde, freqs, phi_ref, m1_SI, m2_SI,
+                                      S1z, S2z, f_ref, shft, r, quadparam1, quadparam2,
+                                      lambda1, lambda2, spinO, tideO, phaseO, amplitudeO);
+
+    XLALDestroyREAL8Sequence(freqs);
 
     *htilde_out = htilde;
-    return XLAL_SUCCESS;
+
+    return ret;
 }
