@@ -37,11 +37,13 @@
 #include <lal/TimeDelay.h>
 #include <lal/SkyCoordinates.h>
 #include <lal/TimeSeries.h>
+#include <lal/TimeSeriesInterp.h>
 #include <lal/FrequencySeries.h>
 #include <lal/TimeFreqFFT.h>
 #include <lal/Window.h>
 #include "check_series_macros.h"
 
+
 /*
  * ============================================================================
  *
@@ -93,9 +95,6 @@ const LALDetector *XLALDetectorPrefixToLALDetector(
 	XLAL_ERROR_NULL(XLAL_EDATA);
 }
 
-/* FIXME:  compatibility wrapper.  remove when not needed */
-const LALDetector *XLALInstrumentNameToLALDetector(const char *string) { return XLALDetectorPrefixToLALDetector(string); }
-
 
 /*
  * ============================================================================
@@ -167,7 +166,7 @@ REAL8TimeSeries * XLALSimQuasiPeriodicInjectionREAL8TimeSeries( REAL8TimeSeries
  *
  * - h+ and hx time series for the injection with their epochs set to the
  * start of those time series at the geocentre (for simplicity the epochs
- * must be the same),
+ * must be the same, and they must have the same length and sample rates),
  *
  * - the right ascension and declination of the source in radians.
  *
@@ -178,16 +177,29 @@ REAL8TimeSeries * XLALSimQuasiPeriodicInjectionREAL8TimeSeries( REAL8TimeSeries
  * Output
  *
  * The strain time series as seen in the detector, with the epoch set to
- * the start of the time series at that detector.
+ * the start of the time series at that detector.  The output time series
+ * units are the same as the two input time series (which must both have
+ * the same sample units).
  *
  * Notes
  *
- * Antenna response factors are computed sample-by-sample, but waveform is
- * not Doppler corrected or otherwise re-interpolated to account for
- * detector motion.
+ * A 19-sample Welch-windowed sinc kernel is used for sub-sample
+ * interpolation.  See XLALREAL8TimeSeriesInterpEval() for more
+ * information, and consider the frequency response of this kernel when
+ * using this function with injections whose frequency content approaches
+ * the Nyquist frequency.
  *
- * The output time series units are the same as the two input time series
- * (which must both have the same sample units).
+ * The geometric delay and antenna response are only recalculated every 250
+ * ms.  The Earth rotates through 7e-5 rad/s, therefore given a radius of
+ * 6e6 m and c=3e8 m/s, the maximum geometric speed for points on the
+ * surface is about 1.5 us/s.  Updating the detector response and geometric
+ * delay every 250 ms means the antenna response is accurate to about +/-
+ * 20 urad and the geometric delay to about +/- 300 ns (about 0.01 sample
+ * at 32 kHz).  The mapping from UTC to sidereal time is only accurate to
+ * +/- 900 ms, so assuming the Earth's orientation to be fixed for 250 ms
+ * at a time is not the dominant source of Earth orientation error in these
+ * calculations, but one should be aware of the periodic nature of the
+ * updates if extreme phase stability is required.
  */
 
 
@@ -197,11 +209,21 @@ REAL8TimeSeries *XLALSimDetectorStrainREAL8TimeSeries(
 	REAL8 right_ascension,
 	REAL8 declination,
 	REAL8 psi,
-	LALDetector *detector
+	const LALDetector *detector
 )
 {
-	char name[13];	/* "?? injection" + terminator */
-	REAL8TimeSeries *h;
+	/* samples */
+	const int kernel_length = 19;
+	/* 0.25 s or 1 sample whichever is larger */
+	const unsigned det_resp_interval = round(0.25 / hplus->deltaT) < 1 ? 1 : round(.25 / hplus->deltaT);
+	LALREAL8TimeSeriesInterp *hplusinterp = NULL;
+	LALREAL8TimeSeriesInterp *hcrossinterp = NULL;
+	double fplus = XLAL_REAL8_FAIL_NAN;
+	double fcross = XLAL_REAL8_FAIL_NAN;
+	double geometric_delay = XLAL_REAL8_FAIL_NAN;
+	double dt;	/* an offset */
+	char *name;
+	REAL8TimeSeries *h = NULL;
 	unsigned i;
 
 	LAL_CHECK_VALID_SERIES(hplus, NULL);
@@ -210,35 +232,82 @@ REAL8TimeSeries *XLALSimDetectorStrainREAL8TimeSeries(
 
 	/* generate name */
 
-	sprintf(name, "%2s injection", detector->frDetector.prefix);
+	name = XLALMalloc(strlen(detector->frDetector.prefix) + 11);
+	if(!name)
+		goto error;
+	sprintf(name, "%s injection", detector->frDetector.prefix);
 
-	/* allocate output time series. */
+	/* allocate output time series.  include 2 and bit times the radius
+	 * of the Earth to accomodate Doppler-induced dilation of the
+	 * waveform */
 
-	h = XLALCreateREAL8TimeSeries(name, &hplus->epoch, hplus->f0, hplus->deltaT, &hplus->sampleUnits, hplus->data->length);
+	h = XLALCreateREAL8TimeSeries(name, &hplus->epoch, hplus->f0, hplus->deltaT, &hplus->sampleUnits, hplus->data->length + (unsigned) ceil(2.0625 * LAL_REARTH_SI / LAL_C_SI / hplus->deltaT));
+	XLALFree(name);
 	if(!h)
-		XLAL_ERROR_NULL(XLAL_EFUNC);
+		goto error;
 
-	/* add the detector's geometric delay.  after this, epoch = the
-	 * time of the injection time series' first sample at the desired
-	 * detector */
+	/* add the detector's geometric delay.  round epoch to an integer
+	 * sample boundary so that XLALSimAddInjectionREAL8TimeSeries() can
+	 * use no-op code path.  note:  we assume a sample boundary occurs
+	 * on the integer second.  if this isn't the case (e.g, some GEO
+	 * data) that's OK, but we might end up paying for a second
+	 * sub-sample time shift when adding the the time series into the
+	 * target data stream in XLALSimAddInjectionREAL8TimeSeries() */
 
-	XLALGPSAdd(&h->epoch, XLALTimeDelayFromEarthCenter(detector->location, right_ascension, declination, &h->epoch));
+	dt = XLALTimeDelayFromEarthCenter(detector->location, right_ascension, declination, &h->epoch);
+	if(XLAL_IS_REAL8_FAIL_NAN(dt))
+		goto error;
+	XLALGPSAdd(&h->epoch, dt);
+	dt = XLALGPSModf(&dt, &h->epoch);
+	XLALGPSAdd(&h->epoch, round(dt / h->deltaT) * h->deltaT - dt);
 
 	/* project + and x time series onto detector */
 
+	hplusinterp = XLALREAL8TimeSeriesInterpCreate(hplus, kernel_length);
+	hcrossinterp = XLALREAL8TimeSeriesInterpCreate(hcross, kernel_length);
+	if(!hplusinterp || !hcrossinterp)
+		goto error;
+
 	for(i = 0; i < h->data->length; i++) {
+		/* time of sample in detector */
 		LIGOTimeGPS t = h->epoch;
-		double fplus, fcross;
-
 		XLALGPSAdd(&t, i * h->deltaT);
-		XLALComputeDetAMResponse(&fplus, &fcross, (const REAL4(*)[3])detector->response, right_ascension, declination, psi, XLALGreenwichMeanSiderealTime(&t));
 
-		h->data->data[i] = fplus * hplus->data->data[i] + fcross * hcross->data->data[i];
+		/* detector's response and geometric delay from geocentre
+		 * at that time */
+		if(!(i % det_resp_interval)) {
+			XLALComputeDetAMResponse(&fplus, &fcross, (const REAL4(*)[3])detector->response, right_ascension, declination, psi, XLALGreenwichMeanSiderealTime(&t));
+			geometric_delay = -XLALTimeDelayFromEarthCenter(detector->location, right_ascension, declination, &t);
+		}
+		if(XLAL_IS_REAL8_FAIL_NAN(fplus) || XLAL_IS_REAL8_FAIL_NAN(fcross) || XLAL_IS_REAL8_FAIL_NAN(geometric_delay))
+			goto error;
+
+		/* time of sample at geocentre.  skip if outside of input
+		 * domain */
+		XLALGPSAdd(&t, geometric_delay);
+		dt = XLALGPSDiff(&t, &hplus->epoch);
+		if(dt < 0.0 || dt >= hplus->data->length * hplus->deltaT) {
+			h->data->data[i] = 0.0;
+			continue;
+		}
+
+		/* evaluate linear combination of interpolators */
+		h->data->data[i] = fplus * XLALREAL8TimeSeriesInterpEval(hplusinterp, &t) + fcross * XLALREAL8TimeSeriesInterpEval(hcrossinterp, &t);
+		if(XLAL_IS_REAL8_FAIL_NAN(h->data->data[i]))
+			goto error;
 	}
+	XLALREAL8TimeSeriesInterpDestroy(hplusinterp);
+	XLALREAL8TimeSeriesInterpDestroy(hcrossinterp);
 
 	/* done */
 
 	return h;
+
+error:
+	XLALREAL8TimeSeriesInterpDestroy(hplusinterp);
+	XLALREAL8TimeSeriesInterpDestroy(hcrossinterp);
+	XLALDestroyREAL8TimeSeries(h);
+	XLAL_ERROR_NULL(XLAL_EFUNC);
 }
 
 
@@ -265,15 +334,13 @@ int XLALSimAddInjectionREAL8TimeSeries(
 	const COMPLEX16FrequencySeries *response
 )
 {
-	COMPLEX16FrequencySeries *tilde_h;
-	REAL8FFTPlan *plan;
-	REAL8Window *window;
+	/* 1 ns is about 10^-5 samples at 16384 Hz */
+	const double noop_threshold = 1e-4;	/* samples */
 	/* the source time series is padded with at least this many 0's at
 	 * the start and end before re-interpolation in an attempt to
 	 * suppress aperiodicity artifacts, and 1/2 this many samples is
 	 * clipped from the start and end afterwards */
 	const unsigned aperiodicity_suppression_buffer = 32768;
-	unsigned i;
 	double start_sample_int;
 	double start_sample_frac;
 
@@ -289,20 +356,6 @@ int XLALSimAddInjectionREAL8TimeSeries(
 		XLAL_ERROR(XLAL_EINVAL);
 	}
 
-	/* extend the source time series by adding the "aperiodicity
-	 * padding" to the start and end.  for efficiency's sake, make sure
-	 * the new length is a power of two. */
-
-	i = round_up_to_power_of_two(h->data->length + 2 * aperiodicity_suppression_buffer);
-	if(i < h->data->length) {
-		/* integer overflow */
-		XLALPrintError("%s(): error: source time series too long\n", __func__);
-		XLAL_ERROR(XLAL_EBADLEN);
-	}
-	i -= h->data->length;
-	if(!XLALResizeREAL8TimeSeries(h, -(int) (i / 2), h->data->length + i))
-		XLAL_ERROR(XLAL_EFUNC);
-
 	/* compute the integer and fractional parts of the sample index in
 	 * the target time series on which the source time series begins.
 	 * modf() returns integer and fractional parts that have the same
@@ -321,137 +374,161 @@ int XLALSimAddInjectionREAL8TimeSeries(
 		start_sample_int += 1.0;
 	}
 
-	/* transform source time series to frequency domain.  the FFT
-	 * function populates the frequency series' metadata with the
-	 * appropriate values. */
+	/* perform sub-sample interpolation if needed */
 
-	tilde_h = XLALCreateCOMPLEX16FrequencySeries(NULL, &h->epoch, 0, 0, &lalDimensionlessUnit, h->data->length / 2 + 1);
-	plan = XLALCreateForwardREAL8FFTPlan(h->data->length, 0);
-	if(!tilde_h || !plan) {
-		XLALDestroyCOMPLEX16FrequencySeries(tilde_h);
+	if(fabs(start_sample_frac) > noop_threshold || response) {
+		COMPLEX16FrequencySeries *tilde_h;
+		REAL8FFTPlan *plan;
+		REAL8Window *window;
+		unsigned i;
+
+		/* extend the source time series by adding the "aperiodicity
+		 * padding" to the start and end.  for efficiency's sake, make sure
+		 * the new length is a power of two, and don't forget to adjust the
+		 * start index in the target time series. */
+
+		i = round_up_to_power_of_two(h->data->length + 2 * aperiodicity_suppression_buffer);
+		if(i < h->data->length) {
+			/* integer overflow */
+			XLALPrintError("%s(): error: source time series too long\n", __func__);
+			XLAL_ERROR(XLAL_EBADLEN);
+		}
+		start_sample_int -= (i - h->data->length) / 2;
+		if(!XLALResizeREAL8TimeSeries(h, -(int) (i - h->data->length) / 2, i))
+			XLAL_ERROR(XLAL_EFUNC);
+
+		/* transform source time series to frequency domain.  the FFT
+		 * function populates the frequency series' metadata with the
+		 * appropriate values. */
+
+		tilde_h = XLALCreateCOMPLEX16FrequencySeries(NULL, &h->epoch, 0, 0, &lalDimensionlessUnit, h->data->length / 2 + 1);
+		plan = XLALCreateForwardREAL8FFTPlan(h->data->length, 0);
+		if(!tilde_h || !plan) {
+			XLALDestroyCOMPLEX16FrequencySeries(tilde_h);
+			XLALDestroyREAL8FFTPlan(plan);
+			XLAL_ERROR(XLAL_EFUNC);
+		}
+		i = XLALREAL8TimeFreqFFT(tilde_h, h, plan);
 		XLALDestroyREAL8FFTPlan(plan);
-		XLAL_ERROR(XLAL_EFUNC);
-	}
-	i = XLALREAL8TimeFreqFFT(tilde_h, h, plan);
-	XLALDestroyREAL8FFTPlan(plan);
-	if(i) {
-		XLALDestroyCOMPLEX16FrequencySeries(tilde_h);
-		XLAL_ERROR(XLAL_EFUNC);
-	}
-
-	/* apply sub-sample time correction and optional response function
-	 * */
-
-	for(i = 0; i < tilde_h->data->length; i++) {
-		const double f = tilde_h->f0 + i * tilde_h->deltaF;
-		COMPLEX16 fac;
-
-		/* phase for sub-sample time correction */
-
-		fac = cexp(-I * LAL_TWOPI * f * start_sample_frac * target->deltaT);
-
-		/* divide the source by the response function.  if a
-		 * frequency is required that lies outside the domain of
-		 * definition of the response function, then the response
-		 * is assumed equal to its value at the nearest edge of the
-		 * domain of definition.  within the domain of definition,
-		 * frequencies are rounded to the nearest bin.  if the
-		 * response function is zero in some bin, then the source
-		 * data is zeroed in that bin (instead of dividing by 0).
-		 * */
-
-		/* FIXME:  should we use GSL to construct an interpolator
-		 * for the modulus and phase as functions of frequency, and
-		 * use that to evaluate the response?  instead of rounding
-		 * to nearest bin? */
-
-		if(response) {
-			int j = floor((f - response->f0) / response->deltaF + 0.5);
-			if(j < 0)
-				j = 0;
-			else if((unsigned) j > response->data->length - 1)
-				j = response->data->length - 1;
-			if(response->data->data[j] == 0.0)
-				fac = 0.0;
-			else
-				fac /= response->data->data[j];
+		if(i) {
+			XLALDestroyCOMPLEX16FrequencySeries(tilde_h);
+			XLAL_ERROR(XLAL_EFUNC);
 		}
 
-		/* apply factor */
+		/* apply sub-sample time correction and optional response function
+		 * */
 
-		tilde_h->data->data[i] *= fac;
-	}
+		for(i = 0; i < tilde_h->data->length; i++) {
+			const double f = tilde_h->f0 + i * tilde_h->deltaF;
+			COMPLEX16 fac;
 
-	/* adjust DC and Nyquist components.  the DC component must always
-	 * be real-valued.  because we have adjusted the source time series
-	 * to have a length that is an even integer (we've made it a power
-	 * of 2) the Nyquist component must also be real valued. */
+			/* phase for sub-sample time correction */
 
-	if(response) {
-		/* a response function has been provided.  zero the DC and
-		 * Nyquist components */
-		if(tilde_h->f0 == 0.0)
-			tilde_h->data->data[0] = 0.0;
-		tilde_h->data->data[tilde_h->data->length - 1] = 0.0;
-	} else {
-		/* no response has been provided.  set the phase of the DC
-		 * component to 0, set the imaginary component of the
-		 * Nyquist to 0 */
-		if(tilde_h->f0 == 0.0)
-			tilde_h->data->data[0] = cabs(tilde_h->data->data[0]);
-		tilde_h->data->data[tilde_h->data->length - 1] = creal(tilde_h->data->data[tilde_h->data->length - 1]);
-	}
+			fac = cexp(-I * LAL_TWOPI * f * start_sample_frac * target->deltaT);
 
-	/* return to time domain */
+			/* divide the source by the response function.  if a
+			 * frequency is required that lies outside the domain of
+			 * definition of the response function, then the response
+			 * is assumed equal to its value at the nearest edge of the
+			 * domain of definition.  within the domain of definition,
+			 * frequencies are rounded to the nearest bin.  if the
+			 * response function is zero in some bin, then the source
+			 * data is zeroed in that bin (instead of dividing by 0).
+			 * */
 
-	plan = XLALCreateReverseREAL8FFTPlan(h->data->length, 0);
-	if(!plan) {
+			/* FIXME:  should we use GSL to construct an interpolator
+			 * for the modulus and phase as functions of frequency, and
+			 * use that to evaluate the response?  instead of rounding
+			 * to nearest bin? */
+
+			if(response) {
+				int j = floor((f - response->f0) / response->deltaF + 0.5);
+				if(j < 0)
+					j = 0;
+				else if((unsigned) j > response->data->length - 1)
+					j = response->data->length - 1;
+				if(response->data->data[j] == 0.0)
+					fac = 0.0;
+				else
+					fac /= response->data->data[j];
+			}
+
+			/* apply factor */
+
+			tilde_h->data->data[i] *= fac;
+		}
+
+		/* adjust DC and Nyquist components.  the DC component must always
+		 * be real-valued.  because we have adjusted the source time series
+		 * to have a length that is an even integer (we've made it a power
+		 * of 2) the Nyquist component must also be real valued. */
+
+		if(response) {
+			/* a response function has been provided.  zero the DC and
+			 * Nyquist components */
+			if(tilde_h->f0 == 0.0)
+				tilde_h->data->data[0] = 0.0;
+			tilde_h->data->data[tilde_h->data->length - 1] = 0.0;
+		} else {
+			/* no response has been provided.  set the phase of the DC
+			 * component to 0, set the imaginary component of the
+			 * Nyquist to 0 */
+			if(tilde_h->f0 == 0.0)
+				tilde_h->data->data[0] = cabs(tilde_h->data->data[0]);
+			tilde_h->data->data[tilde_h->data->length - 1] = creal(tilde_h->data->data[tilde_h->data->length - 1]);
+		}
+
+		/* return to time domain */
+
+		plan = XLALCreateReverseREAL8FFTPlan(h->data->length, 0);
+		if(!plan) {
+			XLALDestroyCOMPLEX16FrequencySeries(tilde_h);
+			XLAL_ERROR(XLAL_EFUNC);
+		}
+		i = XLALREAL8FreqTimeFFT(h, tilde_h, plan);
+		XLALDestroyREAL8FFTPlan(plan);
 		XLALDestroyCOMPLEX16FrequencySeries(tilde_h);
-		XLAL_ERROR(XLAL_EFUNC);
+		if(i)
+			XLAL_ERROR(XLAL_EFUNC);
+
+		/* the deltaT can get "corrupted" by floating point round-off
+		 * during its trip through the frequency domain.  since this
+		 * function starts by confirming that the sample rate of the source
+		 * matches that of the target time series, we can use the target
+		 * series' sample rate to reset the source's sample rate to its
+		 * original value.  but we do a check to make sure we're not
+		 * masking a real bug */
+
+		if(fabs(h->deltaT - target->deltaT) / target->deltaT > 1e-12) {
+			XLALPrintError("%s(): error: oops, internal sample rate mismatch\n", __func__);
+			XLAL_ERROR(XLAL_EERR);
+		}
+		h->deltaT = target->deltaT;
+
+		/* set source epoch from target epoch and integer sample offset */
+
+		h->epoch = target->epoch;
+		XLALGPSAdd(&h->epoch, start_sample_int * target->deltaT);
+
+		/* clip half of the "aperiodicity padding" from the start and end
+		 * of the source time series in a continuing effort to suppress
+		 * aperiodicity artifacts. */
+
+		if(!XLALResizeREAL8TimeSeries(h, aperiodicity_suppression_buffer / 2, h->data->length - aperiodicity_suppression_buffer))
+			XLAL_ERROR(XLAL_EFUNC);
+
+		/* apply a Tukey window whose tapers lie within the remaining
+		 * aperiodicity padding. leaving one sample of the aperiodicty
+		 * padding untouched on each side of the original time series
+		 * because the data might have been shifted into it */
+
+		window = XLALCreateTukeyREAL8Window(h->data->length, (double) (aperiodicity_suppression_buffer - 2) / h->data->length);
+		if(!window)
+			XLAL_ERROR(XLAL_EFUNC);
+		for(i = 0; i < h->data->length; i++)
+			h->data->data[i] *= window->data->data[i];
+		XLALDestroyREAL8Window(window);
 	}
-	i = XLALREAL8FreqTimeFFT(h, tilde_h, plan);
-	XLALDestroyREAL8FFTPlan(plan);
-	XLALDestroyCOMPLEX16FrequencySeries(tilde_h);
-	if(i)
-		XLAL_ERROR(XLAL_EFUNC);
-
-	/* the deltaT can get "corrupted" by floating point round-off
-	 * during its trip through the frequency domain.  since this
-	 * function starts by confirming that the sample rate of the source
-	 * matches that of the target time series, we can use the target
-	 * series' sample rate to reset the source's sample rate to its
-	 * original value.  but we do a check to make sure we're not
-	 * masking a real bug */
-
-	if(fabs(h->deltaT - target->deltaT) / target->deltaT > 1e-12) {
-		XLALPrintError("%s(): error: oops, internal sample rate mismatch\n", __func__);
-		XLAL_ERROR(XLAL_EERR);
-	}
-	h->deltaT = target->deltaT;
-
-	/* set source epoch from target epoch and integer sample offset */
-
-	h->epoch = target->epoch;
-	XLALGPSAdd(&h->epoch, start_sample_int * target->deltaT);
-
-	/* clip half of the "aperiodicity padding" from the start and end
-	 * of the source time series in a continuing effort to suppress
-	 * aperiodicity artifacts. */
-
-	if(!XLALResizeREAL8TimeSeries(h, aperiodicity_suppression_buffer / 2, h->data->length - aperiodicity_suppression_buffer))
-		XLAL_ERROR(XLAL_EFUNC);
-
-	/* apply a Tukey window whose tapers lie within the remaining
-	 * aperiodicity padding. leaving one sample of the aperiodicty
-	 * padding untouched on each side of the original time series
-	 * because the data might have been shifted into it */
-
-	window = XLALCreateTukeyREAL8Window(h->data->length, (double) (aperiodicity_suppression_buffer - 2) / h->data->length);
-	if(!window)
-		XLAL_ERROR(XLAL_EFUNC);
-	for(i = 0; i < h->data->length; i++)
-		h->data->data[i] *= window->data->data[i];
-	XLALDestroyREAL8Window(window);
 
 	/* add source time series to target time series */
 
@@ -488,15 +565,13 @@ int XLALSimAddInjectionREAL4TimeSeries(
 	const COMPLEX8FrequencySeries *response
 )
 {
-	COMPLEX8FrequencySeries *tilde_h;
-	REAL4FFTPlan *plan;
-	REAL4Window *window;
+	/* 1 ns is about 10^-5 samples at 16384 Hz */
+	const double noop_threshold = 1e-4;	/* samples */
 	/* the source time series is padded with at least this many 0's at
 	 * the start and end before re-interpolation in an attempt to
 	 * suppress aperiodicity artifacts, and 1/2 this many samples is
 	 * clipped from the start and end afterwards */
 	const unsigned aperiodicity_suppression_buffer = 32768;
-	unsigned i;
 	double start_sample_int;
 	double start_sample_frac;
 
@@ -512,20 +587,6 @@ int XLALSimAddInjectionREAL4TimeSeries(
 		XLAL_ERROR(XLAL_EINVAL);
 	}
 
-	/* extend the source time series by adding the "aperiodicity
-	 * padding" to the start and end.  for efficiency's sake, make sure
-	 * the new length is a power of two. */
-
-	i = round_up_to_power_of_two(h->data->length + 2 * aperiodicity_suppression_buffer);
-	if(i < h->data->length) {
-		/* integer overflow */
-		XLALPrintError("%s(): error: source time series too long\n", __func__);
-		XLAL_ERROR(XLAL_EBADLEN);
-	}
-	i -= h->data->length;
-	if(!XLALResizeREAL4TimeSeries(h, -(int) (i / 2), h->data->length + i))
-		XLAL_ERROR(XLAL_EFUNC);
-
 	/* compute the integer and fractional parts of the sample index in
 	 * the target time series on which the source time series begins.
 	 * modf() returns integer and fractional parts that have the same
@@ -544,137 +605,161 @@ int XLALSimAddInjectionREAL4TimeSeries(
 		start_sample_int += 1.0;
 	}
 
-	/* transform source time series to frequency domain.  the FFT
-	 * function populates the frequency series' metadata with the
-	 * appropriate values. */
+	/* perform sub-sample interpolation if needed */
 
-	tilde_h = XLALCreateCOMPLEX8FrequencySeries(NULL, &h->epoch, 0, 0, &lalDimensionlessUnit, h->data->length / 2 + 1);
-	plan = XLALCreateForwardREAL4FFTPlan(h->data->length, 0);
-	if(!tilde_h || !plan) {
-		XLALDestroyCOMPLEX8FrequencySeries(tilde_h);
+	if(fabs(start_sample_frac) > noop_threshold || response) {
+		COMPLEX8FrequencySeries *tilde_h;
+		REAL4FFTPlan *plan;
+		REAL4Window *window;
+		unsigned i;
+
+		/* extend the source time series by adding the "aperiodicity
+		 * padding" to the start and end.  for efficiency's sake, make sure
+		 * the new length is a power of two, and don't forget to adjust the
+		 * start index in the target time series. */
+
+		i = round_up_to_power_of_two(h->data->length + 2 * aperiodicity_suppression_buffer);
+		if(i < h->data->length) {
+			/* integer overflow */
+			XLALPrintError("%s(): error: source time series too long\n", __func__);
+			XLAL_ERROR(XLAL_EBADLEN);
+		}
+		start_sample_int -= (i - h->data->length) / 2;
+		if(!XLALResizeREAL4TimeSeries(h, -(int) (i - h->data->length) / 2, i))
+			XLAL_ERROR(XLAL_EFUNC);
+
+		/* transform source time series to frequency domain.  the FFT
+		 * function populates the frequency series' metadata with the
+		 * appropriate values. */
+
+		tilde_h = XLALCreateCOMPLEX8FrequencySeries(NULL, &h->epoch, 0, 0, &lalDimensionlessUnit, h->data->length / 2 + 1);
+		plan = XLALCreateForwardREAL4FFTPlan(h->data->length, 0);
+		if(!tilde_h || !plan) {
+			XLALDestroyCOMPLEX8FrequencySeries(tilde_h);
+			XLALDestroyREAL4FFTPlan(plan);
+			XLAL_ERROR(XLAL_EFUNC);
+		}
+		i = XLALREAL4TimeFreqFFT(tilde_h, h, plan);
 		XLALDestroyREAL4FFTPlan(plan);
-		XLAL_ERROR(XLAL_EFUNC);
-	}
-	i = XLALREAL4TimeFreqFFT(tilde_h, h, plan);
-	XLALDestroyREAL4FFTPlan(plan);
-	if(i) {
-		XLALDestroyCOMPLEX8FrequencySeries(tilde_h);
-		XLAL_ERROR(XLAL_EFUNC);
-	}
-
-	/* apply sub-sample time correction and optional response function
-	 * */
-
-	for(i = 0; i < tilde_h->data->length; i++) {
-		const double f = tilde_h->f0 + i * tilde_h->deltaF;
-		COMPLEX8 fac;
-
-		/* phase for sub-sample time correction */
-
-		fac = cexp(-I * LAL_TWOPI * f * start_sample_frac * target->deltaT);
-
-		/* divide the source by the response function.  if a
-		 * frequency is required that lies outside the domain of
-		 * definition of the response function, then the response
-		 * is assumed equal to its value at the nearest edge of the
-		 * domain of definition.  within the domain of definition,
-		 * frequencies are rounded to the nearest bin.  if the
-		 * response function is zero in some bin, then the source
-		 * data is zeroed in that bin (instead of dividing by 0).
-		 * */
-
-		/* FIXME:  should we use GSL to construct an interpolator
-		 * for the modulus and phase as functions of frequency, and
-		 * use that to evaluate the response?  instead of rounding
-		 * to nearest bin? */
-
-		if(response) {
-			int j = floor((f - response->f0) / response->deltaF + 0.5);
-			if(j < 0)
-				j = 0;
-			else if((unsigned) j > response->data->length - 1)
-				j = response->data->length - 1;
-			if(response->data->data[j] == 0.0)
-				fac = 0.0;
-			else
-				fac /= response->data->data[j];
+		if(i) {
+			XLALDestroyCOMPLEX8FrequencySeries(tilde_h);
+			XLAL_ERROR(XLAL_EFUNC);
 		}
 
-		/* apply factor */
+		/* apply sub-sample time correction and optional response function
+		 * */
 
-		tilde_h->data->data[i] *= fac;
-	}
+		for(i = 0; i < tilde_h->data->length; i++) {
+			const double f = tilde_h->f0 + i * tilde_h->deltaF;
+			COMPLEX8 fac;
 
-	/* adjust DC and Nyquist components.  the DC component must always
-	 * be real-valued.  because we have adjusted the source time series
-	 * to have a length that is an even integer (we've made it a power
-	 * of 2) the Nyquist component must also be real valued. */
+			/* phase for sub-sample time correction */
 
-	if(response) {
-		/* a response function has been provided.  zero the DC and
-		 * Nyquist components */
-		if(tilde_h->f0 == 0.0)
-			tilde_h->data->data[0] = 0.0;
-		tilde_h->data->data[tilde_h->data->length - 1] = 0.0;
-	} else {
-		/* no response has been provided.  set the phase of the DC
-		 * component to 0, set the imaginary component of the
-		 * Nyquist to 0 */
-		if(tilde_h->f0 == 0.0)
-			tilde_h->data->data[0] = cabsf(tilde_h->data->data[0]);
-		tilde_h->data->data[tilde_h->data->length - 1] = crealf(tilde_h->data->data[tilde_h->data->length - 1]);
-	}
+			fac = cexp(-I * LAL_TWOPI * f * start_sample_frac * target->deltaT);
 
-	/* return to time domain */
+			/* divide the source by the response function.  if a
+			 * frequency is required that lies outside the domain of
+			 * definition of the response function, then the response
+			 * is assumed equal to its value at the nearest edge of the
+			 * domain of definition.  within the domain of definition,
+			 * frequencies are rounded to the nearest bin.  if the
+			 * response function is zero in some bin, then the source
+			 * data is zeroed in that bin (instead of dividing by 0).
+			 * */
 
-	plan = XLALCreateReverseREAL4FFTPlan(h->data->length, 0);
-	if(!plan) {
+			/* FIXME:  should we use GSL to construct an interpolator
+			 * for the modulus and phase as functions of frequency, and
+			 * use that to evaluate the response?  instead of rounding
+			 * to nearest bin? */
+
+			if(response) {
+				int j = floor((f - response->f0) / response->deltaF + 0.5);
+				if(j < 0)
+					j = 0;
+				else if((unsigned) j > response->data->length - 1)
+					j = response->data->length - 1;
+				if(response->data->data[j] == 0.0)
+					fac = 0.0;
+				else
+					fac /= response->data->data[j];
+			}
+
+			/* apply factor */
+
+			tilde_h->data->data[i] *= fac;
+		}
+
+		/* adjust DC and Nyquist components.  the DC component must always
+		 * be real-valued.  because we have adjusted the source time series
+		 * to have a length that is an even integer (we've made it a power
+		 * of 2) the Nyquist component must also be real valued. */
+
+		if(response) {
+			/* a response function has been provided.  zero the DC and
+			 * Nyquist components */
+			if(tilde_h->f0 == 0.0)
+				tilde_h->data->data[0] = 0.0;
+			tilde_h->data->data[tilde_h->data->length - 1] = 0.0;
+		} else {
+			/* no response has been provided.  set the phase of the DC
+			 * component to 0, set the imaginary component of the
+			 * Nyquist to 0 */
+			if(tilde_h->f0 == 0.0)
+				tilde_h->data->data[0] = cabsf(tilde_h->data->data[0]);
+			tilde_h->data->data[tilde_h->data->length - 1] = crealf(tilde_h->data->data[tilde_h->data->length - 1]);
+		}
+
+		/* return to time domain */
+
+		plan = XLALCreateReverseREAL4FFTPlan(h->data->length, 0);
+		if(!plan) {
+			XLALDestroyCOMPLEX8FrequencySeries(tilde_h);
+			XLAL_ERROR(XLAL_EFUNC);
+		}
+		i = XLALREAL4FreqTimeFFT(h, tilde_h, plan);
+		XLALDestroyREAL4FFTPlan(plan);
 		XLALDestroyCOMPLEX8FrequencySeries(tilde_h);
-		XLAL_ERROR(XLAL_EFUNC);
+		if(i)
+			XLAL_ERROR(XLAL_EFUNC);
+
+		/* the deltaT can get "corrupted" by floating point round-off
+		 * during its trip through the frequency domain.  since this
+		 * function starts by confirming that the sample rate of the source
+		 * matches that of the target time series, we can use the target
+		 * series' sample rate to reset the source's sample rate to its
+		 * original value.  but we do a check to make sure we're not
+		 * masking a real bug */
+
+		if(fabs(h->deltaT - target->deltaT) / target->deltaT > 1e-12) {
+			XLALPrintError("%s(): error: oops, internal sample rate mismatch\n", __func__);
+			XLAL_ERROR(XLAL_EERR);
+		}
+		h->deltaT = target->deltaT;
+
+		/* set source epoch from target epoch and integer sample offset */
+
+		h->epoch = target->epoch;
+		XLALGPSAdd(&h->epoch, start_sample_int * target->deltaT);
+
+		/* clip half of the "aperiodicity padding" from the start and end
+		 * of the source time series in a continuing effort to suppress
+		 * aperiodicity artifacts. */
+
+		if(!XLALResizeREAL4TimeSeries(h, aperiodicity_suppression_buffer / 2, h->data->length - aperiodicity_suppression_buffer))
+			XLAL_ERROR(XLAL_EFUNC);
+
+		/* apply a Tukey window whose tapers lie within the remaining
+		 * aperiodicity padding. leaving one sample of the aperiodicty
+		 * padding untouched on each side of the original time series
+		 * because the data might have been shifted into it */
+
+		window = XLALCreateTukeyREAL4Window(h->data->length, (double) (aperiodicity_suppression_buffer - 2) / h->data->length);
+		if(!window)
+			XLAL_ERROR(XLAL_EFUNC);
+		for(i = 0; i < h->data->length; i++)
+			h->data->data[i] *= window->data->data[i];
+		XLALDestroyREAL4Window(window);
 	}
-	i = XLALREAL4FreqTimeFFT(h, tilde_h, plan);
-	XLALDestroyREAL4FFTPlan(plan);
-	XLALDestroyCOMPLEX8FrequencySeries(tilde_h);
-	if(i)
-		XLAL_ERROR(XLAL_EFUNC);
-
-	/* the deltaT can get "corrupted" by floating point round-off
-	 * during its trip through the frequency domain.  since this
-	 * function starts by confirming that the sample rate of the source
-	 * matches that of the target time series, we can use the target
-	 * series' sample rate to reset the source's sample rate to its
-	 * original value.  but we do a check to make sure we're not
-	 * masking a real bug */
-
-	if(fabs(h->deltaT - target->deltaT) / target->deltaT > 1e-12) {
-		XLALPrintError("%s(): error: oops, internal sample rate mismatch\n", __func__);
-		XLAL_ERROR(XLAL_EERR);
-	}
-	h->deltaT = target->deltaT;
-
-	/* set source epoch from target epoch and integer sample offset */
-
-	h->epoch = target->epoch;
-	XLALGPSAdd(&h->epoch, start_sample_int * target->deltaT);
-
-	/* clip half of the "aperiodicity padding" from the start and end
-	 * of the source time series in a continuing effort to suppress
-	 * aperiodicity artifacts. */
-
-	if(!XLALResizeREAL4TimeSeries(h, aperiodicity_suppression_buffer / 2, h->data->length - aperiodicity_suppression_buffer))
-		XLAL_ERROR(XLAL_EFUNC);
-
-	/* apply a Tukey window whose tapers lie within the remaining
-	 * aperiodicity padding. leaving one sample of the aperiodicty
-	 * padding untouched on each side of the original time series
-	 * because the data might have been shifted into it */
-
-	window = XLALCreateTukeyREAL4Window(h->data->length, (double) (aperiodicity_suppression_buffer - 2) / h->data->length);
-	if(!window)
-		XLAL_ERROR(XLAL_EFUNC);
-	for(i = 0; i < h->data->length; i++)
-		h->data->data[i] *= window->data->data[i];
-	XLALDestroyREAL4Window(window);
 
 	/* add source time series to target time series */
 
