@@ -20,11 +20,11 @@
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <getopt.h>
 
 #include <gsl/gsl_rng.h>
 
 #include <lal/LALStdlib.h>
+#include <lal/LALgetopt.h>
 #include <lal/LALConstants.h>
 #include <lal/Date.h>
 #include <lal/Units.h>
@@ -96,7 +96,7 @@ end:
 
 int parseargs( int argc, char **argv )
 {
-	struct option long_options[] = {
+	struct LALoption long_options[] = {
 			{ "help", no_argument, 0, 'h' },
 			{ "geo", no_argument, 0, 'G' },
 			{ "hanford", no_argument, 0, 'H' },
@@ -114,7 +114,7 @@ int parseargs( int argc, char **argv )
 		int option_index = 0;
 		int c;
 	
-		c = getopt_long_only(argc, argv, args, long_options, &option_index);
+		c = LALgetopt_long_only(argc, argv, args, long_options, &option_index);
 		if (c == -1) /* end of options */
 			break;
 	
@@ -123,7 +123,7 @@ int parseargs( int argc, char **argv )
 				if (long_options[option_index].flag)
 					break;
 				else {
-					fprintf(stderr, "error parsing option %s with argument %s\n", long_options[option_index].name, optarg);
+					fprintf(stderr, "error parsing option %s with argument %s\n", long_options[option_index].name, LALoptarg);
 					exit(1);
 				}
 			case 'h': /* help */
@@ -142,19 +142,19 @@ int parseargs( int argc, char **argv )
 				detectors[numDetectors++] = lalCachedDetectors[LAL_VIRGO_DETECTOR];
 				break;
 			case 's': /* start-time */
-				tstart = atof(optarg);
+				tstart = atof(LALoptarg);
 				break;
 			case 't': /* duration */
-				duration = atof(optarg);
+				duration = atof(LALoptarg);
 				break;
 			case 'r': /* sample-rate */
-				srate = atof(optarg);
+				srate = atof(LALoptarg);
 				break;
 			case 'W': /* Omega0 */
-				Omega0 = atof(optarg);
+				Omega0 = atof(LALoptarg);
 				break;
 			case 'f': /* low-frequency */
-				flow = atof(optarg);
+				flow = atof(LALoptarg);
 				break;
 			case '?':
 			default:
@@ -163,10 +163,10 @@ int parseargs( int argc, char **argv )
 		}
 	}
 	
-	if ( optind < argc ) {
+	if ( LALoptind < argc ) {
 		fprintf(stderr, "extraneous command line arguments:\n");
-		while (optind < argc)
-			fprintf(stderr, "%s\n", argv[optind++]);
+		while (LALoptind < argc)
+			fprintf(stderr, "%s\n", argv[LALoptind++]);
 		exit(1);
 	}
 
