@@ -86,16 +86,22 @@ const char * usage =
 "                             EOBNRv2HM\n"
 "                             SEOBNRv1\n"
 "                             SEOBNRv2\n"
+"                             SEOBNRv3\n"
 "                             SpinTaylorT4\n"
 "                             SpinTaylorT2\n"
 "                             PhenSpinTaylor\n"
 "                             PhenSpinTaylorRD\n"
 "                             SpinDominatedWf\n"
+"                             HGimri\n"
 "                           Supported FD approximants:\n"
 "                             IMRPhenomA\n"
 "                             IMRPhenomB\n"
 "                             IMRPhenomC\n"
 "                             IMRPhenomP\n"
+"                             SEOBNRv1_ROM_SingleSpin\n"
+"                             SEOBNRv1_ROM_DoubleSpin\n"
+"                             SEOBNRv2_ROM_SingleSpin\n"
+"                             SEOBNRv2_ROM_DoubleSpin\n"
 "                             TaylorF2\n"
 "                             SpinTaylorF2\n"
 "                             TaylorR2F4\n"
